from confz import BaseConfig, FileSource
from pydantic import SecretStr, AnyUrl

from pydantic import BaseModel, Field
from typing import Optional, List

class AiderConfigSchema(BaseConfig):
    # Main configuration
    openai_api_key: Optional[SecretStr] = Field(None, description="Specify the OpenAI API key")
    anthropic_api_key: Optional[SecretStr] = Field(None, description="Specify the Anthropic API key")
    model: Optional[str] = Field(None, description="Specify the model to use for the main chat")
    opus: bool = Field(False, description="Use claude-3-opus-20240229 model for the main chat")
    sonnet: bool = Field(False, description="Use claude-3-5-sonnet-20240620 model for the main chat")
    model_4: bool = Field(False, description="Use gpt-4-0613 model for the main chat")
    model_4o: bool = Field(False, description="Use gpt-4o-2024-08-06 model for the main chat")
    mini: bool = Field(False, description="Use gpt-4o-mini model for the main chat")
    model_4_turbo: bool = Field(False, description="Use gpt-4-1106-preview model for the main chat")
    model_35turbo: bool = Field(False, description="Use gpt-3.5-turbo model for the main chat")
    deepseek: bool = Field(False, description="Use deepseek/deepseek-coder model for the main chat")

    # Model settings
    list_models: Optional[bool] = Field(None, description="List known models which match the (partial) MODEL name")
    openai_api_base: Optional[str] = Field(None, description="Specify the OpenAI API base URL")
    openai_api_type: Optional[str] = Field(None, description="Specify the OpenAI API type")
    openai_api_version: Optional[str] = Field(None, description="Specify the OpenAI API version")
    openai_api_deployment_id: Optional[str] = Field(None, description="Specify the deployment ID")
    openai_organization_id: Optional[str] = Field(None, description="Specify the OpenAI organization ID")
    model_settings_file: Optional[str] = Field(None, description="Specify a file with aider model settings for unknown models")
    model_metadata_file: Optional[str] = Field(None, description="Specify a file with context window and costs for unknown models")
    verify_ssl: bool = Field(True, description="Verify the SSL cert when connecting to models")
    edit_format: Optional[str] = Field(None, description="Specify what edit format the LLM should use")
    weak_model: Optional[str] = Field(None, description="Specify the model to use for commit messages and chat history summarization")
    show_model_warnings: bool = Field(True, description="Only work with models that have meta-data available")
    map_tokens: int = Field(1024, description="Suggested number of tokens to use for repo map")
    map_refresh: Optional[str] = Field("auto", description="Control how often the repo map is refreshed")
    cache_prompts: bool = Field(False, description="Enable caching of prompts")
    cache_keepalive_pings: int = Field(0, description="Number of times to ping at 5min intervals to keep prompt cache warm")
    map_multiplier_no_files: bool = Field(True, description="Multiplier for map tokens when no files are specified")
    max_chat_history_tokens: Optional[int] = Field(None, description="Maximum number of tokens to use for chat history")
    env_file: str = Field(".env", description="Specify the .env file to load")

    # History files
    input_history_file: str = Field(".aider.input.history", description="Specify the chat input history file")
    chat_history_file: str = Field(".aider.chat.history.md", description="Specify the chat history file")
    restore_chat_history: bool = Field(False, description="Restore the previous chat history messages")
    llm_history_file: Optional[str] = Field(None, description="Log the conversation with the LLM to this file")

    # Output settings
    dark_mode: bool = Field(False, description="Use colors suitable for a dark terminal background")
    light_mode: bool = Field(False, description="Use colors suitable for a light terminal background")
    pretty: bool = Field(True, description="Enable/disable pretty, colorized output")
    stream: bool = Field(True, description="Enable/disable streaming responses")
    user_input_color: str = Field("#00cc00", description="Set the color for user input")
    tool_output_color: Optional[str] = Field(None, description="Set the color for tool output")
    tool_error_color: str = Field("#FF2222", description="Set the color for tool error messages")
    tool_warning_color: str = Field("#FFA500", description="Set the color for tool warning messages")
    assistant_output_color: str = Field("#0088ff", description="Set the color for assistant output")
    code_theme: str = Field("default", description="Set the markdown code theme")
    show_diffs: bool = Field(False, description="Show diffs when committing changes")

    # Git settings
    git: bool = Field(True, description="Enable/disable looking for a git repo")
    gitignore: bool = Field(True, description="Enable/disable adding .aider* to .gitignore")
    aiderignore: Optional[str] = Field(None, description="Specify the aider ignore file")
    subtree_only: bool = Field(False, description="Only consider files in the current subtree of the git repository")
    auto_commits: bool = Field(True, description="Enable/disable auto commit of LLM changes")
    dirty_commits: bool = Field(True, description="Enable/disable commits when repo is found dirty")
    attribute_author: bool = Field(True, description="Attribute aider code changes in the git author name")
    attribute_committer: bool = Field(True, description="Attribute aider commits in the git committer name")
    attribute_commit_message_author: bool = Field(False, description="Prefix commit messages with 'aider: ' if aider authored the changes")
    attribute_commit_message_committer: bool = Field(False, description="Prefix all commit messages with 'aider: '")
    commit: bool = Field(False, description="Commit all pending changes with a suitable commit message, then exit")
    commit_prompt: Optional[str] = Field(None, description="Specify a custom prompt for generating commit messages")
    dry_run: bool = Field(False, description="Perform a dry run without modifying files")

    # Fixing and committing
    lint: bool = Field(False, description="Lint and fix provided files, or dirty files if none provided")
    lint_cmd: Optional[List[str]] = Field(None, description="Specify lint commands to run for different languages")
    auto_lint: bool = Field(True, description="Enable/disable automatic linting after changes")
    test_cmd: Optional[str] = Field(None, description="Specify command to run tests")
    auto_test: bool = Field(False, description="Enable/disable automatic testing after changes")
    test: bool = Field(False, description="Run tests and fix problems found")

    # Other settings
    file: Optional[List[str]] = Field(None, description="Specify a file to edit")
    read: Optional[List[str]] = Field(None, description="Specify a read-only file")
    vim: bool = Field(False, description="Use VI editing mode in the terminal")
    voice_language: str = Field("en", description="Specify the language for voice using ISO 639-1 code")
    chat_language: Optional[str] = Field(None, description="Specify the language to use in the chat")
    version: Optional[bool] = Field(None, description="Show the version number and exit")
    just_check_update: bool = Field(False, description="Check for updates and return status in the exit code")
    check_update: bool = Field(True, description="Check for new aider versions on launch")
    install_main_branch: bool = Field(False, description="Install the latest version from the main branch")
    upgrade: bool = Field(False, description="Upgrade aider to the latest version from PyPI")
    apply: Optional[str] = Field(None, description="Apply the changes from the given file instead of running the chat")
    yes: bool = Field(False, description="Always say yes to every confirmation")
    verbose: bool = Field(False, description="Enable verbose output")
    show_repo_map: bool = Field(False, description="Print the repo map and exit")
    show_prompts: bool = Field(False, description="Print the system prompts and exit")
    exit: bool = Field(False, description="Do all startup activities then exit before accepting user input")
    message: Optional[str] = Field(None, description="Specify a single message to send the LLM, process reply then exit")
    message_file: Optional[str] = Field(None, description="Specify a file containing the message to send the LLM, process reply, then exit")
    encoding: str = Field("utf-8", description="Specify the encoding for input and output")
    config: Optional[str] = Field(None, description="Specify the config file")
    gui: bool = Field(False, description="Run aider in your browser")
    suggest_shell_commands: bool = Field(True, description="Enable/disable suggesting shell commands")

    CONFIG_SOURCES = FileSource(file='.aider.conf.yml')







import inspect
from functools import partial, wraps

import asyncer
from typer import Typer


class AsyncTyper(Typer):
    @staticmethod
    def maybe_run_async(decorator, f):
        if inspect.iscoroutinefunction(f):

            @wraps(f)
            def runner(*args, **kwargs):
                return asyncer.runnify(f)(*args, **kwargs)

            decorator(runner)
        else:
            decorator(f)
        return f

    def callback(self, *args, **kwargs):
        decorator = super().callback(*args, **kwargs)
        return partial(self.maybe_run_async, decorator)

    def command(self, *args, **kwargs):
        decorator = super().command(*args, **kwargs)
        return partial(self.maybe_run_async, decorator)



from fastapi import FastAPI, APIRouter, HTTPException, Body
from typing import Type, Dict, Any
from dslmodel import DSLModel, init_lm
import inspect

app = FastAPI()  # Initialize FastAPI

def auto_router(model_class: Type[DSLModel]):
    """
    Decorator to automatically register REST API routes for a DSLModel class.
    """
    router = APIRouter()
    model_name = model_class.__name__.lower()

    # Create endpoint to generate a model instance from prompt
    @router.post(f"/{model_name}/")
    async def create_model_instance(data: Dict[str, Any] = Body(...)):
        init_lm()  # Initialize the language model
        try:
            instance = model_class.from_prompt(**data)
            return instance.model_dump()
        except Exception as e:
            raise HTTPException(status_code=400, detail=str(e))

    # Create endpoint to return the model's JSON schema
    @router.get(f"/{model_name}/schema")
    async def get_model_schema():
        try:
            return model_class.model_json_schema()
        except Exception as e:
            raise HTTPException(status_code=500, detail=str(e))

    # Register the router with the FastAPI app
    app.include_router(router)

    # Return the original class to maintain decorator behavior
    return model_class

from pydantic import Field

@auto_router
class Participant(DSLModel):
    name: str = Field(..., description="Name of the participant.")
    role: str = Field(..., description="Role of the participant.")


import uvicorn
import threading
from dslmodel import init_lm, init_instant, init_text

def main():
    """Main function to initialize the app and start the server."""
    # Initialize the necessary DSL model components
    init_instant()

    # Define a function to run the FastAPI app
    def run_app():
        uvicorn.run(app, host="0.0.0.0", port=8000)

    # Start the FastAPI server in a separate thread
    thread = threading.Thread(target=run_app, daemon=True)
    thread.start()

    # Keep the main thread alive to prevent immediate exit
    print("Server is running... Press Ctrl+C to exit.")
    try:
        thread.join()  # Wait for the server thread to finish
    except KeyboardInterrupt:
        print("Shutting down the server...")

if __name__ == '__main__':
    main()




"""
This command presents the concept of a service colony and its characteristics. A service colony is a novel architectural style for developing a software system as a group of autonomous software services co-operating to fulfill the objectives of the system. Each inhabitant service in the colony implements a specific system functionality, collaborates with the other services, and makes proactive decisions that impact its performance and interaction patterns with other inhabitants. By increasing the level of self-awareness and autonomy available to individual system components, the resulting system is increasingly more decentralized, distributed, flexible, adaptable, distributed, modular, robust, and fault-tolerant.
"""
import typer


app = typer.Typer()


@app.command(name="inhabitant")
def _inhabitant():
    """inhabitant"""
    typer.echo("Running inhabitant subcommand.")
     
@app.command(name="add")
def inhabitant_add():
    """add"""
    typer.echo("Running add subcommand.")
 
@app.command(name="remove")
def inhabitant_remove():
    """remove"""
    typer.echo("Running remove subcommand.")
 
@app.command(name="list")
def inhabitant_list():
    """list"""
    typer.echo("Running list subcommand.")
 
@app.command(name="start")
def inhabitant_start():
    """start"""
    typer.echo("Running start subcommand.")
 
@app.command(name="stop")
def inhabitant_stop():
    """stop"""
    typer.echo("Running stop subcommand.")
 
@app.command(name="logs")
def inhabitant_logs():
    """logs"""
    typer.echo("Running logs subcommand.")



import os
import shutil

import dspy
import typer
from pathlib import Path
from jinja2 import Template
from typing import Optional
import inflection
import json

from sungen.lm.cerebras import Cerebras
from sungen.utils.file_tools import plugins_dir
from sungen.utils.plugin_tools import (
    PluginSettings,
    DependencyConfig,
    CompatibilityConfig,
    MarketplaceConfig,
    SupportConfig,
    AdvancedOptionsConfig,
    create_plugin_yaml,
    load_plugin_config,
)
from sungen.utils.chat_tools import chatbot
from sungen.utils.dspy_tools import init_dspy, init_ol, GPT_DEFAULT_MODEL
from io import StringIO

from sungen.utils.str_tools import pythonic_str

app = typer.Typer(name="plugin")

# Templates for plugin files
plugin_py_template = '''"""
{{ plugin_name }} Plugin
This plugin provides {{ plugin_description }}.

Author: {{ author }}
Version: {{ version }}
"""

import typer


app = typer.Typer(name="{{ plugin_name }}", help="{{ plugin_description }}")


@app.command()
def example():
    """
    Example command for {{ plugin_name }}.
    
    Usage:
    sungen {{ plugin_name }} example
    """
    typer.echo("This is an example command for the {{ plugin_name }} plugin.")


def check_installation():
    """
    Check if {{ plugin_name }} dependencies are installed and install them if not present.
    """
    try:
        import importlib.util
        import subprocess
        import sys


        if importlib.util.find_spec("{{ plugin_name }}_dep") is None:
            print("{{ plugin_name }}_dep is not installed. Installing...")
            subprocess.check_call([sys.executable, "-m", "pip", "install", "{{ plugin_name }}_dep"])
            print("{{ plugin_name }}_dep has been installed successfully.")

    except Exception as e:
        print(f"Error during library installation check")
        raise e


def register_plugin(parent_app: typer.Typer):
    """Register the plugin with the main application."""
    # check_installation()

    parent_app.add_typer(app, name="{{ plugin_name }}")
'''


def create_directory_structure(plugin_name: str, base_dir: Optional[Path] = None) -> Path:
    """Create the directory structure for a new plugin."""
    pythonic_name = inflection.underscore(plugin_name)
    if base_dir is None:
        base_dir = Path.cwd() / "src" / "plugins"
    base_path = base_dir / pythonic_name
    base_path.mkdir(parents=True, exist_ok=True)
    return base_path


def render_template(template: str, **kwargs) -> str:
    """Render a template using Jinja2."""
    return Template(template).render(**kwargs)


def write_file(filepath: Path, content: str) -> None:
    """Write content to a file."""
    filepath.write_text(content)


def generate_plugin_files(plugin_settings: PluginSettings, base_dir: Optional[Path] = None) -> None:
    """Generates the necessary files for a new plugin."""
    base_path = create_directory_structure(plugin_settings.name, base_dir)
    pythonic_name = inflection.underscore(plugin_settings.name)

    # Render the Python plugin file
    plugin_py_content = render_template(
        plugin_py_template,
        plugin_name=plugin_settings.name,
        plugin_description=plugin_settings.description,
        author=plugin_settings.author,
        version=plugin_settings.version
    )
    write_file(base_path / f"{pythonic_name}_plugin.py", plugin_py_content)

    # Create the YAML configuration file
    create_plugin_yaml(plugin_settings, str(base_path / f"plugin.yaml"))

    # Create an empty __init__.py file
    write_file(base_path / "__init__.py", "")


def modify_files(target_path: Path, plugin_name: str) -> None:
    """
    Modify or add necessary files in the target project after migration.

    Args:
        target_path (Path): The path where the plugins have been copied.
        plugin_name (str): The name of the plugin being migrated.
    """
    try:
        # Example: Update or add necessary files in the target project
        plugin_file = target_path / f"{plugin_name}_plugin.py"

        if plugin_file.exists():
            # Read the content to modify it if necessary
            content = plugin_file.read_text()
            # Modify the content if needed
            # This is a placeholder modification
            content += "\n# Additional configurations or custom code here\n"
            plugin_file.write_text(content)

        # Add or modify any other files as needed
        typer.echo(f"Modified files in {target_path} for plugin '{plugin_name}'.")
    except Exception as e:
        typer.echo(f"Error modifying files: {str(e)}")
        raise e

def copy_plugin_files(source_plugin_path: Path, target_plugin_path: Path) -> None:
    """
    Copy plugin files from the source to the target directory.

    Args:
        source_plugin_path (Path): The source directory of the plugins.
        target_plugin_path (Path): The target directory where the plugins will be moved.
    """
    try:
        if not target_plugin_path.exists():
            target_plugin_path.mkdir(parents=True)

        for item in source_plugin_path.iterdir():
            s = source_plugin_path / item
            d = target_plugin_path / item.name

            if s.is_dir():
                shutil.copytree(s, d, dirs_exist_ok=True)
            else:
                shutil.copy2(s, d)

        typer.echo(f"Successfully migrated plugins from {source_plugin_path} to {target_plugin_path}.")
    except Exception as e:
        typer.echo(f"Error during plugin migration: {str(e)}")
        raise e

@app.command()
def migrate(
    target_projects: list[str] = typer.Argument(..., help="List of paths to the target projects"),
    local_plugins_path: Path = typer.Option(Path("src/sungen/plugins"), help="Path to the local plugins directory"),
    remove_old: bool = typer.Option(False, "--remove-old", help="Remove old plugin files after migration"),
):
    """
    Migrate plugins from the local development directory to specified target projects.

    Args:
        target_projects (list[str]): Paths to the target project directories.
        local_plugins_path (Path): The source directory containing the local plugins.
        remove_old (bool): Whether to remove old plugin files after migration.
    """
    if not local_plugins_path.exists() or not local_plugins_path.is_dir():
        typer.echo(f"Local plugins path '{local_plugins_path}' does not exist or is not a directory.")
        raise typer.Exit(code=1)

    for project_path in target_projects:
        target_path = Path(project_path) / "src" / "sungen" / "plugins"

        # Copy the plugin files to the target directory
        copy_plugin_files(local_plugins_path, target_path)

        # Get all plugins from the local path and modify their target versions
        for plugin_dir in local_plugins_path.iterdir():
            if plugin_dir.is_dir():
                modify_files(target_path / plugin_dir.name, plugin_dir.name)

        if remove_old:
            shutil.rmtree(local_plugins_path)
            typer.echo(f"Old plugins removed from {local_plugins_path}.")


@app.command(name="create")
def create_plugin(
        plugin_name: str = typer.Option(..., prompt=True, help="Name of the plugin"),
        description: str = typer.Option(..., prompt=True, help="Description of the plugin"),
        author: str = typer.Option(..., prompt=True, help="Author of the plugin"),
        version: str = typer.Option("1.0.0", prompt=True, help="Version of the plugin"),
        license: str = typer.Option("MIT", prompt=True, help="License for the plugin"),
        min_sungen_version: Optional[str] = typer.Option("1.0.0", help="Minimum compatible Sungen version"),
        max_sungen_version: Optional[str] = typer.Option("2.0.0", help="Maximum compatible Sungen version"),
        base_dir: Optional[Path] = typer.Option(None, help="Base directory for plugin creation"),
):
    """
    CLI Command: Creates a new plugin with the specified name and metadata.

    sungen plugin create --plugin-name "Interpolation" --description "This plugin performs text interpolation." --author "Sean Chatman" --version "1.0.0" --license "MIT" --min-sungen-version "1.0.0" --max-sungen-version "2.0.0"
    """
    pythonic_name = inflection.underscore(plugin_name)

    if base_dir is None:
        base_dir = plugins_dir()

    plugin_settings = PluginSettings(
        name=pythonic_name,
        short_name=pythonic_name[:3],
        version=version,
        description=description,
        author=author,
        source=f"https://github.com/sungen/{pythonic_name}",
        license=license,
        tags=[pythonic_name],
        settings={
            "default_marketplace": "https://marketplace.sungen.com",
            "cache_timeout": 300,
            "auto_update_check": True,
            "log_level": "INFO",
            "retry_count": 3,
            "connection_timeout": 30,
            "backup_before_update": True,
        },
        dependencies=[
            DependencyConfig(plugin="core", version=">=1.0.0"),
        ],
        compatibility=CompatibilityConfig(
            min_sungen_version=min_sungen_version,
            max_sungen_version=max_sungen_version
        ),
        marketplace=MarketplaceConfig(
            featured=False,
            category="Utilities",
            keywords=[pythonic_name]
        ),
        support=SupportConfig(
            documentation_url=f"https://docs.sungen.com/{pythonic_name}",
            issues_url=f"https://github.com/sungen/{pythonic_name}/issues",
            contact_email="support@sungen.com"
        ),
        advanced=AdvancedOptionsConfig(
            parallel_install=True,
            secure_downloads=True,
            sandbox_mode=False
        )
    )

    generate_plugin_files(plugin_settings, base_dir)
    typer.echo(f"Plugin '{plugin_name}' created successfully!")


class GeneratePluginInfo(dspy.Signature):
    """
    Generate the name and description for a plugin based on the provided prompt.
    """

    prompt = dspy.InputField(desc="Text prompt containing details or requirements for the plugin.")

    reasoning = dspy.OutputField(desc="Brief rationale for the plugin's design.")
    plugin_name = dspy.OutputField(desc="A concise name for the plugin (2 words, <8 chars).")
    plugin_description = dspy.OutputField(desc="A brief description of the plugin's purpose and functionality.")


default_author: str | None = os.getenv("PLUGIN_AUTHOR", None)
do_prompt = os.getenv("PLUGIN_AUTHOR", None) is None


@app.command(name="prompt")
def _prompt(
        prompt: str = typer.Argument(..., help="Prompt describing the functionality and purpose of the plugin."),
        author: str = typer.Option(
            default=default_author,
            prompt=do_prompt,
            help="Author of the plugin"
        ),
        base_dir: Optional[Path] = typer.Option(plugins_dir(), help="Base directory for plugin creation")
):
    """
    CLI Command: Generate a new plugin based on a prompt

    Example:
    sungen generate-plugin --prompt "Plugin that performs advanced text interpolation for dynamic content." --author "John Doe"
    """
    # Your existing code logic
    # Step 1: Create an instance of the GeneratePluginInfo Signature class

    # Step 2: Use the DSPy Predictor to get the plugin name and description
    init_dspy(lm_class=Cerebras, max_tokens=2000)

    predictor = dspy.Predict(GeneratePluginInfo)
    output = predictor(prompt=prompt)

    # Step 3: Extract generated plugin name and description
    plugin_name = pythonic_str(output.plugin_name)
    plugin_description = output.plugin_description

    # Define default values for other plugin settings
    version = "1.0.0"
    license_type = "MIT"
    min_sungen_version = "1.0.0"
    max_sungen_version = "2.0.0"

    # Step 4: Generate Plugin Settings
    plugin_settings = PluginSettings(
        name=plugin_name,
        short_name=plugin_name[:3],
        version=version,
        description=plugin_description,
        author=author,
        source=f"https://github.com/sungen/{inflection.underscore(plugin_name)}",
        license=license_type,
        tags=[plugin_name],
        settings={
            "default_marketplace": "https://marketplace.sungen.com",
            "cache_timeout": 300,
            "auto_update_check": True,
            "log_level": "INFO",
            "retry_count": 3,
            "connection_timeout": 30,
            "backup_before_update": True,
        },
        dependencies=[
            DependencyConfig(plugin="core", version=">=1.0.0"),
        ],
        compatibility=CompatibilityConfig(
            min_sungen_version=min_sungen_version,
            max_sungen_version=max_sungen_version
        ),
        marketplace=MarketplaceConfig(
            featured=False,
            category="Utilities",
            keywords=[inflection.underscore(plugin_name)]
        ),
        support=SupportConfig(
            documentation_url=f"https://docs.sungen.com/{inflection.underscore(plugin_name)}",
            issues_url=f"https://github.com/sungen/{inflection.underscore(plugin_name)}/issues",
            contact_email="support@sungen.com"
        ),
        advanced=AdvancedOptionsConfig(
            parallel_install=True,
            secure_downloads=True,
            sandbox_mode=False
        )
    )

    # Step 5: Generate Plugin Files
    generate_plugin_files(plugin_settings, base_dir)
    typer.echo(f"Plugin '{plugin_name}' created successfully!")


@app.command(name="chat")
def chat(
    question: str = typer.Argument(..., help="Question for the chatbot about plugin_cmd.py"),
    model: str = typer.Option(GPT_DEFAULT_MODEL, help="Model to use for the chatbot (e.g., gpt-4-turbo, qwen2:instruct)")
):
    """
    CLI Command: Chat about the plugin_cmd.py file using the specified model.
    """
    # Get the content of plugin_cmd.py
    plugin_cmd_path = Path(__file__)
    plugin_cmd_content = plugin_cmd_path.read_text()
    

    
    # Prepare the context for the chatbot
    context = f"""
    File: plugin_cmd.py
    
    File Content:
    {plugin_cmd_content}
    """
    
    # Start the chat session
    typer.echo(f"Starting chat session about plugin_cmd.py using model: {model}...")
    history = chatbot(question=question, context=context, model=model)
    
    while True:
        user_input = typer.prompt("You", default="exit")
        if user_input.lower() in ['exit', 'quit', 'q']:
            break
        history = chatbot(question=user_input, context=context, history=history, model=model)

    typer.echo("Chat session ended.")


if __name__ == "__main__":
    app()



# """Meta commands for sungen that combine multiple commands and plugins."""

# import typer
# import subprocess
# from pathlib import Path
# from sungen.cmds import plugin_cmd, templates_cmd
# from sungen.utils.plugin_tools import load_plugins
# from sungen.utils.cli_tools import load_commands, source_dir
# from sungen.utils.chat_tools import chatbot
# from sungen.utils.echo_utils import echo
# import shutil
# import os
# import json
# from datetime import datetime
# import ast
# import cProfile
# import pstats
# import io

# app = typer.Typer()

# # Load all available commands and plugins
# load_commands(app, source_dir("cmds"))
# load_plugins(app)

# @app.command()
# def create_full_plugin(
#     plugin_name: str = typer.Option(..., prompt=True, help="Name of the plugin"),
#     description: str = typer.Option(..., prompt=True, help="Description of the plugin"),
#     author: str = typer.Option(..., prompt=True, help="Author of the plugin"),
# ):
#     """Create a new plugin with all necessary files and templates."""
#     echo(f"Creating full plugin: {plugin_name}", style="bold green", panel=True, title="Plugin Creation")

#     # Create the basic plugin structure
#     plugin_cmd.create_plugin(
#         plugin_name=plugin_name,
#         description=description,
#         author=author,
#     )

#     # Generate plugin-specific templates
#     templates_cmd.plugin_tools(f"Create plugin tools for {plugin_name}")
#     templates_cmd.plugin_cmd(f"Create plugin command for {plugin_name}")
#     templates_cmd.test_plugin_cmd(f"Create test for {plugin_name} command")
#     templates_cmd.test_plugin_tools(f"Create test for {plugin_name} tools")

#     echo(f"Full plugin {plugin_name} created successfully!", style="bold green")

# @app.command()
# def update_all_templates():
#     """Update all templates with the latest configurations and best practices."""
#     echo("Updating all templates...", style="bold blue", panel=True, title="Template Update")

#     templates_cmd.chat_tools("Update chat tools template with latest best practices")
#     templates_cmd.plugin_tools("Update plugin tools template with latest best practices")
#     templates_cmd.dspy_tools("Update DSPy tools template with latest best practices")
#     templates_cmd.cli("Update CLI template with latest best practices")
#     templates_cmd.cli_tools("Update CLI tools template with latest best practices")
#     templates_cmd.config("Update config template with latest best practices")
#     templates_cmd.plugin_cmd("Update plugin command template with latest best practices")
#     templates_cmd.test_plugin_cmd("Update test plugin command template with latest best practices")
#     templates_cmd.test_plugin_tools("Update test plugin tools template with latest best practices")
#     templates_cmd.readme("Update README template with latest best practices")

#     echo("All templates updated successfully!", style="bold green")

# @app.command()
# def setup_development_environment():
#     """Set up a complete development environment with necessary plugins and configurations."""
#     echo("Setting up development environment...", style="bold magenta", panel=True, title="Dev Environment Setup")

#     # Create essential plugins
#     create_full_plugin(
#         plugin_name="core",
#         description="Core functionality for the project",
#         author="Sungen Team",
#     )
#     create_full_plugin(
#         plugin_name="dev_tools",
#         description="Development tools and utilities",
#         author="Sungen Team",
#     )

#     # Update all templates
#     update_all_templates()

#     # Additional setup steps
#     echo("Installing development dependencies...", style="yellow")
#     # Add code to install dev dependencies (e.g., using poetry or pip)

#     echo("Configuring pre-commit hooks...", style="yellow")
#     # Add code to set up pre-commit hooks

#     echo("Development environment setup complete!", style="bold green")

# @app.command()
# def run_full_test_suite():
#     """Run all tests, including unit tests, integration tests, and plugin tests."""
#     echo("Running full test suite...", style="bold cyan", panel=True, title="Test Suite")

#     # Run unit tests
#     echo("Running unit tests...", style="cyan")
#     # Add code to run unit tests (e.g., using pytest)

#     # Run integration tests
#     echo("Running integration tests...", style="cyan")
#     # Add code to run integration tests

#     # Run plugin tests
#     echo("Running plugin tests...", style="cyan")
#     # Iterate through all plugins and run their tests

#     echo("Full test suite completed!", style="bold green")

# @app.command()
# def generate_project_structure(
#     project_name: str = typer.Option(..., prompt=True, help="Name of the project"),
#     template: str = typer.Option("default", help="Project structure template to use"),
# ):
#     """Generate a complete project structure based on a template."""
#     echo(f"Generating project structure for {project_name} using {template} template...", style="bold yellow", panel=True, title="Project Structure Generation")
    
#     base_dir = Path(project_name)
#     base_dir.mkdir(exist_ok=True)
    
#     # Create directory structure
#     dirs = [
#         "src", "tests", "docs", "config",
#         "src/models", "src/utils", "src/services",
#         "tests/unit", "tests/integration"
#     ]
#     for dir_name in dirs:
#         (base_dir / dir_name).mkdir(parents=True, exist_ok=True)
    
#     # Create basic files
#     (base_dir / "README.md").write_text(f"# {project_name}\n\nAdd project description here.")
#     (base_dir / "requirements.txt").touch()
#     (base_dir / ".gitignore").write_text("__pycache__\n*.pyc\n.venv\n")
#     (base_dir / "src/__init__.py").touch()
#     (base_dir / "tests/__init__.py").touch()
    
#     echo("Project structure generated successfully!", style="bold green")

# @app.command()
# def analyze_codebase(
#     path: Path = typer.Option(Path.cwd(), help="Path to the codebase to analyze"),
# ):
#     """Analyze the codebase and provide insights on code quality, structure, and potential improvements."""
#     echo(f"Analyzing codebase at {path}...", style="bold blue", panel=True, title="Codebase Analysis")
    
#     total_lines = 0
#     total_files = 0
#     class_count = 0
#     function_count = 0
    
#     for root, _, files in os.walk(path):
#         for file in files:
#             if file.endswith('.py'):
#                 total_files += 1
#                 file_path = os.path.join(root, file)
#                 with open(file_path, 'r') as f:
#                     content = f.read()
#                     total_lines += len(content.splitlines())
#                     tree = ast.parse(content)
#                     class_count += sum(isinstance(node, ast.ClassDef) for node in ast.walk(tree))
#                     function_count += sum(isinstance(node, ast.FunctionDef) for node in ast.walk(tree))
    
#     echo(f"Total Python files: {total_files}")
#     echo(f"Total lines of code: {total_lines}")
#     echo(f"Total classes: {class_count}")
#     echo(f"Total functions: {function_count}")
    
#     # Run pylint
#     # pylint_output = io.StringIO()
#     # echo("Pylint report:", style="bold")
#     # echo(pylint_output.getvalue())
    
#     echo("Codebase analysis complete.", style="green")

# @app.command()
# def generate_documentation(
#     path: Path = typer.Option(Path.cwd(), help="Path to the project"),
#     output_format: str = typer.Option("markdown", help="Output format for documentation"),
# ):
#     """Generate comprehensive documentation for the project."""
#     echo(f"Generating documentation for project at {path} in {output_format} format...", style="bold magenta", panel=True, title="Documentation Generation")
    
#     docs_dir = path / "docs"
#     docs_dir.mkdir(exist_ok=True)
    
#     for root, _, files in os.walk(path):
#         for file in files:
#             if file.endswith('.py'):
#                 file_path = Path(root) / file
#                 relative_path = file_path.relative_to(path)
#                 output_file = docs_dir / f"{relative_path.with_suffix('.md')}"
#                 output_file.parent.mkdir(parents=True, exist_ok=True)
                
#                 with open(file_path, 'r') as f:
#                     content = f.read()
#                     tree = ast.parse(content)
                    
#                     doc = f"# {file}\n\n"
#                     for node in ast.walk(tree):
#                         if isinstance(node, (ast.FunctionDef, ast.ClassDef)):
#                             doc += f"## {node.name}\n\n"
#                             if ast.get_docstring(node):
#                                 doc += f"{ast.get_docstring(node)}\n\n"
                
#                 output_file.write_text(doc)
    
#     echo("Documentation generated successfully!", style="bold green")

# @app.command()
# def optimize_performance(
#     path: Path = typer.Option(Path.cwd(), help="Path to the project"),
# ):
#     """Analyze and optimize the performance of the project."""
#     echo(f"Optimizing performance for project at {path}...", style="bold yellow", panel=True, title="Performance Optimization")
    
#     main_file = path / "src" / "main.py"
#     if main_file.exists():
#         profiler = cProfile.Profile()
#         profiler.enable()
        
#         # Run the main script
#         exec(main_file.read_text(), globals())
        
#         profiler.disable()
#         stats = pstats.Stats(profiler).sort_stats('cumulative')
        
#         echo("Top 10 time-consuming functions:", style="bold")
#         stats.print_stats(10)
#     else:
#         echo("main.py not found. Please specify the main entry point of your application for profiling.")
    
#     echo("Performance optimization complete!", style="bold green")

# @app.command()
# def generate_test_suite(
#     path: Path = typer.Option(Path.cwd(), help="Path to the project"),
# ):
#     """Generate a comprehensive test suite for the project."""
#     echo(f"Generating test suite for project at {path}...", style="bold cyan", panel=True, title="Test Suite Generation")
    
#     tests_dir = path / "tests"
#     tests_dir.mkdir(exist_ok=True)
    
#     for root, _, files in os.walk(path / "src"):
#         for file in files:
#             if file.endswith('.py') and not file.startswith('test_'):
#                 file_path = Path(root) / file
#                 relative_path = file_path.relative_to(path / "src")
#                 test_file = tests_dir / f"test_{relative_path}"
#                 test_file.parent.mkdir(parents=True, exist_ok=True)
                
#                 with open(file_path, 'r') as f:
#                     content = f.read()
#                     tree = ast.parse(content)
                    
#                     test_content = f"import unittest\nfrom {relative_path.with_suffix('').as_posix().replace('/', '.')} import *\n\n"
#                     test_content += f"class Test{file.capitalize().replace('.py', '')}(unittest.TestCase):\n"
                    
#                     for node in ast.walk(tree):
#                         if isinstance(node, ast.FunctionDef):
#                             test_content += f"    def test_{node.name}(self):\n        self.assertTrue(True)  # TODO: Implement test\n\n"
                
#                 test_file.write_text(test_content)
    
#     echo("Test suite generated successfully!", style="bold green")

# @app.command()
# def deploy_project(
#     environment: str = typer.Option(..., prompt=True, help="Deployment environment (e.g., dev, staging, production)"),
# ):
#     """Deploy the project to the specified environment."""
#     echo(f"Deploying project to {environment} environment...", style="bold red", panel=True, title="Project Deployment")
    
#     # Run tests
#     subprocess.run(["pytest"], check=True)
    
#     # Build the project
#     subprocess.run(["python", "setup.py", "sdist", "bdist_wheel"], check=True)
    
#     # Deploy (simulated)
#     deploy_dir = Path(f"deploy_{environment}")
#     deploy_dir.mkdir(exist_ok=True)
#     for item in Path("dist").glob("*"):
#         shutil.copy(item, deploy_dir)
    
#     # Update configuration
#     config = {
#         "environment": environment,
#         "version": "1.0.0",
#         "deploy_time": datetime.now().isoformat()
#     }
#     with open(deploy_dir / "config.json", "w") as f:
#         json.dump(config, f, indent=2)
    
#     echo(f"Project deployed successfully to {environment}!", style="bold green")

# @app.command()
# def ai_assistant(
#     query: str = typer.Option(..., prompt=True, help="Your question or request for the AI assistant"),
# ):
#     """Interact with an AI assistant for project-related queries and tasks."""
#     echo("Processing your request...", style="bold blue", panel=True, title="AI Assistant")
#     response = chatbot(query)
#     echo(f"AI Assistant: {response}", style="green")

# if __name__ == "__main__":
#     app()


"""support"""
import typer


app = typer.Typer()


@app.command(name="support")
def _support():
    """support"""
    typer.echo("Running support subcommand.")
     
@app.command(name="help")
def support_help():
    """help"""
    typer.echo("Running help subcommand.")
 
@app.command(name="docs")
def support_docs():
    """docs"""
    typer.echo("Running docs subcommand.")
 
@app.command(name="contact")
def support_contact():
    """contact"""
    typer.echo("Running contact subcommand.")
 
@app.command(name="version")
def support_version():
    """version"""
    typer.echo("Running version subcommand.")
 
@app.command(name="feedback")
def support_feedback():
    """feedback"""
    typer.echo("Running feedback subcommand.")



"""Documentation: The source code provided is a Python script that uses the dspy and typer libraries to generate a
command-line interface (CLI) based on a given concept. The script defines a class called GenCLIModule, which contains
a forward method that takes in a CLI concept and uses the dspy library to generate a CLI with commands. The
gen_cli_call function then uses this class to generate the CLI based on a given concept. The script also includes a
main function that initializes the dspy library and calls the gen_cli_call function with an empty CLI concept.
Additionally, the script includes a streamlit component from the fastapi library and a router that defines a post
route for generating the CLI. The gen_cli_route function uses the gen_cli_call function to generate the CLI based on
the data provided in the post request."""
import dspy
from pydantic import BaseModel, Field
from typer import Typer

from dslmodel import DSLModel
from sungen.utils.dspy_tools import init_dspy

app = Typer()


class GenCLIModule(dspy.Module):
    """GenCLIModule"""

    def forward(self, cli_concept):
        # Generate mock CLI help
        pred = dspy.Predict("cli_concept -> cli_with_commands")
        result = pred(cli_concept=cli_concept).cli_with_commands
        return result


def gen_cli_call(cli_concept):
    gen_cli = GenCLIModule()
    return gen_cli.forward(cli_concept=cli_concept)


@app.command()
def call(cli_concept):
    """GenCLIModule"""
    init_dspy()
    
    print(gen_cli_call(cli_concept=cli_concept))


class TyperCommand(BaseModel):
    name: str = Field(..., min_length=1, description="The name of the command")
    help: str = Field(..., min_length=1, description="The help text for the command")


class TyperCLI(DSLModel):
    name: str = Field(..., min_length=1, description="The name of the CLI application")
    commands: list[TyperCommand] = Field(
        ..., description="The commands of the CLI application"
    )


# --- Jinja Templates ---
cli_template = """
import typer
app = typer.Typer()

{% for command in model.commands %}
@app.command(name="{{ command.name }}")
def {{ command.name }}():
    \"\"\"{{ command.help }}\"\"\"
    # Command logic goes here
    print("This is the {{ command.name }} command.")

{% endfor %}

if __name__ == "__main__":
    app()


"""


pytest_template = """
import pytest
from typer.testing import CliRunner
from sungen.cli import app  # Updated import statement

runner = CliRunner()

{% for command in model.commands %}
def test_{{ command.name }}():
    result = runner.invoke(app, ["{{ command.name }}"])
    assert result.exit_code == 0
    assert "This is the {{ command.name }} command." in result.output  # Replace with specific expected output

{% endfor %}
"""


def main():
    init_dspy()

    concept = gen_concept("7 Command Expert Python ChatBot with OpenAI calls")

    print(concept)

    # model = gen_pydantic_instance_call(prompt=concept,
    #     root_model=TyperCLI, child_models=[TyperCommand]
    # )
    #
    # print(model.to_yaml())
    #
    # render(cli_template, model=model, to="{{ model.name | underscore }}_cli.py")
    # render(pytest_template, model=model, to="test_{{ model.name | underscore }}_cli.py")

    # # --- Render Templates ---
    # env = Environment(loader=FileSystemLoader("."))
    # env.from_string(cli_template).stream(model=model.model_dump()).dump(
    #     "ror_dspy.py"
    # )
    # env.from_string(pytest_template).stream(model=model.model_dump()).dump(
    #     "test_ror_dspy.py"
    # )

    print("CLI application and tests generated.")

from fastapi import APIRouter
router = APIRouter()

@router.post("/gen_cli/")
async def gen_cli_route(data: dict):
    # Your code generation logic here
    init_dspy()
    
    print(data)
    return gen_cli_call(**data)


def gen_concept(cli_concept):
    init_dspy()
    style = ("Verbose output that simulates the --help command of the synthetic"
             "CLI.")

    pred = dspy.Predict("cli_concept, style -> synthetic_cli_help")
    result = pred(cli_concept=cli_concept, style=style).synthetic_cli_help
    return result


if __name__ == "__main__":
    main()



import requests
from bs4 import BeautifulSoup


def main():
    """Main function"""
    from sungen.utils.dspy_tools import init_ol
    init_ol()

    # URL of the Ash documentation page
    url = "https://hexdocs.pm/ash/readme.html#how-to"

    # Fetch the content of the page
    response = requests.get(url)

    # Check if the request was successful
    if response.status_code == 200:
        # Parse the HTML content
        soup = BeautifulSoup(response.text, "html.parser")

        # Extract the desired content, such as all 'a' tags (links)
        links = soup.find_all('a')

        # Print the links and their text
        for link in links:
            print(f"Text: {link.text}, URL: {link.get('href')}")
    else:
        print(f"Failed to retrieve the page. Status code: {response.status_code}")


if __name__ == '__main__':
    main()


"""---
contextdocs:
  - name: Ash Framework
    relationship: Core framework for building resources, data layers, and actions
    resources:
      - Official Documentation: https://hexdocs.pm/ash
      - Getting Started Guide: https://hexdocs.pm/ash/get-started.html
      - Ash Resource DSL Reference: https://hexdocs.pm/ash/dsl-ash-resource.html

  - name: AshPostgres
    relationship: Data layer for PostgreSQL, used for persistence and transactions
    resources:
      - Official Documentation: https://hexdocs.pm/ash_postgres
      - PostgreSQL Configuration Guide: https://www.postgresql.org/docs/

  - name: AshOban
    relationship: Background job processing for compliance checks, scheduled tasks, and job orchestration
    resources:
      - Official Documentation: https://hexdocs.pm/ash_oban
      - Oban Documentation: https://hexdocs.pm/oban/Oban.html

  - name: AshReactor
    relationship: Complex workflow orchestration for state machines and multi-step business processes
    resources:
      - Official Documentation: https://hexdocs.pm/ash/reactor.html
      - Reactor DSL Reference: https://hexdocs.pm/ash/dsl-ash-reactor.html
      - Reactor Usage Guide: https://hexdocs.pm/ash_reactor/getting-started.html

  - name: AshStateMachine
    relationship: Used to define state machines for ticket and financial transaction workflows
    resources:
      - Official Documentation: https://hexdocs.pm/ash_state_machine
      - State Machine Concepts: https://en.wikipedia.org/wiki/Finite-state_machine
      - AshStateMachine Cookbook: https://hexdocs.pm/ash_state_machine/cookbook.html

  - name: AshMoney
    relationship: Handles multi-currency operations and provides support for financial calculations
    resources:
      - Official Documentation: https://hexdocs.pm/ash_money
      - ex_money_sql: https://github.com/kipcole9/money_sql

  - name: AshDoubleEntry
    relationship: Implements double-entry accounting system for transaction management
    resources:
      - Official Documentation: https://hexdocs.pm/ash_double_entry
      - Double Entry Accounting Concepts: https://en.wikipedia.org/wiki/Double-entry_bookkeeping_system

  - name: AshPaperTrail
    relationship: Provides auditing and versioning for compliance with regulations like SOX
    resources:
      - Official Documentation: https://hexdocs.pm/ash_paper_trail
      - Paper Trail Concepts: https://en.wikipedia.org/wiki/Paper_trail

  - name: AshCloak
    relationship: Handles encryption and secure storage of sensitive data for compliance with data protection regulations (e.g., GDPR)
    resources:
      - Official Documentation: https://hexdocs.pm/ash_cloak
      - Data Encryption Best Practices: https://www.owasp.org/index.php/Guide_to_Cryptography

  - name: AshAdmin
    relationship: Provides an administrative interface for managing the helpdesk and financial modules
    resources:
      - Official Documentation: https://hexdocs.pm/ash_admin

  - name: AshAppsignal
    relationship: Integrates real-time monitoring and alerting to track system health and compliance events
    resources:
      - Official Documentation: https://hexdocs.pm/ash_appsignal
      - AppSignal Monitoring: https://docs.appsignal.com

  - name: Smokestack
    relationship: Test factories for generating data in tests, improving test coverage and quality
    resources:
      - Official Documentation: https://hexdocs.pm/smokestack
      - Factory Design Pattern: https://refactoring.guru/design-patterns/factory-method

  - name: SWIFT Payment System
    relationship: External integration for processing international financial transactions
    resources:
      - Official Documentation: https://www.swift.com
      - SWIFT Integration Guide: https://www.swift.com/our-solutions

  - name: Node.js
    relationship: Runtime environment for some ancillary tools or scripts, including JavaScript-based monitoring agents
    resources:
      - Official Documentation: https://nodejs.org/en/docs/
      - Best Practices Guide: https://github.com/goldbergyoni/nodebestpractices

  - name: OpenAPI Specification
    relationship: API design standard used for documenting the RESTful and GraphQL APIs
    resources:
      - Official Documentation: https://swagger.io/specification/
      - OpenAPI 3.0 Guide: https://swagger.io/docs/specification/about/
---

# Additional Documentation Notes

This `.contextdocs.md` file provides essential documentation resources for understanding and extending the Financial Ash Ecosystem project. The project leverages multiple Ash Framework extensions to handle various complex tasks such as state management, job scheduling, financial transactions, and compliance auditing. External resources listed here cover both the theoretical foundations and practical implementations of these capabilities.

Regularly review and update these references to ensure they remain accurate and relevant as the project evolves.

/Users/sac/Library/Caches/pypoetry/virtualenvs/stargen-pwn7Ka8j-py3.12/bin/python /Users/sac/dev/sungen/src/sungen/plugins/ash/pull_ash_docs.py 
Text: 

, URL: https://github.com/ash-project/ash
Text: 
ash
        , URL: https://github.com/ash-project/ash
Text: 

View Source
, URL: https://github.com/ash-project/ash/blob/v3.4.11/README.md#L1
Text: , URL: https://opensource.org/licenses/MIT
Text: , URL: https://hex.pm/packages/ash
Text: , URL: https://hexdocs.pm/ash
Text: hexdocs, URL: https://hexdocs.pm/ash
Text: 

, URL: #dive-in
Text: What is Ash?, URL: what-is-ash.html
Text: Get Started, URL: get-started.html
Text: See the roadmap, URL: https://github.com/orgs/ash-project/projects/3
Text: 

, URL: #about-the-documentation
Text: Tutorials, URL: #tutorials
Text: Topics, URL: #topics
Text: How-to, URL: #how-to
Text: Reference, URL: #reference
Text: 

, URL: #tutorials
Text: Get Started, URL: get-started.html
Text: 

, URL: #topics
Text: 

, URL: #about-ash
Text: What is Ash?, URL: what-is-ash.html
Text: Our Design Principles, URL: design-principles.html
Text: Contributing to Ash, URL: contributing-to-ash.html
Text: Alternatives, URL: alternatives.html
Text: 

, URL: #resources
Text: Domains, URL: domains.html
Text: Attributes, URL: attributes.html
Text: Relationships, URL: relationships.html
Text: Calculations, URL: calculations.html
Text: Aggregates, URL: aggregates.html
Text: Code Interfaces, URL: code-interfaces.html
Text: Identities, URL: identities.html
Text: Validations, URL: validations.html
Text: Changes, URL: changes.html
Text: Preparations, URL: preparations.html
Text: Embedded Resources, URL: embedded-resources.html
Text: Notifiers, URL: notifiers.html
Text: 

, URL: #actions
Text: Actions, URL: actions.html
Text: Read Actions, URL: read-actions.html
Text: Create Actions, URL: create-actions.html
Text: Update Actions, URL: update-actions.html
Text: Destroy Actions, URL: destroy-actions.html
Text: Generic Actions, URL: generic-actions.html
Text: Manual Actions, URL: manual-actions.html
Text: 

, URL: #security
Text: Actors & Authorization, URL: actors-and-authorization.html
Text: Sensitive Data, URL: sensitive-data.html
Text: Policies, URL: policies.html
Text: 

, URL: #development
Text: Project Structure, URL: project-structure.html
Text: Generators, URL: generators.html
Text: Testing, URL: testing.html
Text: Development Utilities, URL: development-utilities.html
Text: Upgrading to 3.0, URL: upgrading-to-3-0.html
Text: Error Handling, URL: error-handling.html
Text: 

, URL: #advanced
Text: Monitoring, URL: monitoring.html
Text: Multitenancy, URL: multitenancy.html
Text: Reactor, URL: reactor.html
Text: Timeouts, URL: timeouts.html
Text: Writing Extensions, URL: writing-extensions.html
Text: 

, URL: #how-to
Text: Test Resources, URL: test-resources.html
Text: Authorize Access to Resources, URL: authorize-access-to-resources.html
Text: Encrypt Attributes, URL: encrypt-attributes.html
Text: Prevent Concurrent Writes, URL: prevent-concurrent-writes.html
Text: Wrap External APIs, URL: wrap-external-apis.html
Text: 

, URL: #reference
Text: Glossary, URL: glossary.html
Text: Expressions, URL: expressions.html
Text: Ash.Resource DSL, URL: dsl-ash-resource.html
Text: Ash.Domain DSL, URL: dsl-ash-domain.html
Text: Ash.Reactor DSL, URL: dsl-ash-reactor.html
Text: Ash.Notifier.PubSub DSL, URL: dsl-ash-notifier-pubsub.html
Text: Ash.Policy.Authorizer DSL, URL: dsl-ash-policy-authorizer.html
Text: Ash.DataLayer.Ets DSL, URL: dsl-ash-datalayer-ets.html
Text: Ash.DataLayer.Mnesia DSL, URL: dsl-ash-datalayer-mnesia.html
Text: 

, URL: #packages
Text: 

, URL: #data-layers
Text: AshPostgres, URL: https://hexdocs.pm/ash_postgres
Text: AshSqlite, URL: https://hexdocs.pm/ash_sqlite
Text: AshCsv, URL: https://hexdocs.pm/ash_csv
Text: AshCubdb, URL: https://hexdocs.pm/ash_cubdb
Text: 

, URL: #api-extensions
Text: AshJsonApi, URL: https://hexdocs.pm/ash_json_api
Text: AshGraphql, URL: https://hexdocs.pm/ash_graphql
Text: 

, URL: #web
Text: AshPhoenix, URL: https://hexdocs.pm/ash_phoenix
Text: AshAuthentication, URL: https://hexdocs.pm/ash_authentication
Text: AshAuthenticationPhoenix, URL: https://hexdocs.pm/ash_authentication_phoenix
Text: 

, URL: #finance
Text: AshMoney, URL: https://hexdocs.pm/ash_money
Text: AshDoubleEntry, URL: https://hexdocs.pm/ash_double_entry
Text: 

, URL: #resource-utilities
Text: AshOban, URL: https://hexdocs.pm/ash_oban
Text: AshArchival, URL: https://hexdocs.pm/ash_archival
Text: AshStateMachine, URL: https://hexdocs.pm/ash_state_machine
Text: AshPaperTrail, URL: https://hexdocs.pm/ash_paper_trail
Text: AshCloak, URL: https://hexdocs.pm/ash_cloak
Text: 

, URL: #admin-monitoring
Text: AshAdmin, URL: https://hexdocs.pm/ash_admin
Text: AshAppsignal, URL: https://hexdocs.pm/ash_appsignal
Text: 

, URL: #testing
Text: Smokestack, URL: https://hexdocs.pm/smokestack
Text: 

          â Previous Page
        

API Reference
        
, URL: api-reference.html
Text: 

          Next Page â
        

Get Started
        
, URL: get-started.html
Text: Hex Package, URL: https://hex.pm/packages/ash/3.4.11
Text: Hex Preview, URL: https://preview.hex.pm/preview/ash/3.4.11
Text: current file, URL: https://preview.hex.pm/preview/ash/3.4.11/show/README.md
Text: 
              Download ePub version
            , URL: ash.epub
Text: ExDoc, URL: https://github.com/elixir-lang/ex_doc
Text: Elixir programming language, URL: https://elixir-lang.org

Process finished with exit code 0

"""


"""repo"""
import typer
import os
import subprocess

from sungen.utils.cli_tools import load_commands
from sungen.utils.file_tools import source_dir
from sungen.utils.git_tools import github

app = typer.Typer()

def first_commit(
    name: str,
    commit_message: str = "Initial commit",
    branch: str = "main",
    path: str = "."
):
    """
    Create an initial commit in the specified repository.
    """
    gh = github()
    user = gh.get_user()
    repo = user.get_repo(name)

    # Change to the specified path
    os.chdir(path)

    # Clone the repository
    subprocess.run(["git", "clone", repo.clone_url, name], check=True)
    os.chdir(os.path.join(path, name))  # Change directory to the cloned repo

    # Create a README.md
    with open("README.md", "w") as f:
        f.write(f"# {name}\n\nThis is the {name} repository.")

    # Stage, commit, and push the changes
    subprocess.run(["git", "add", "README.md"], check=True)
    subprocess.run(["git", "commit", "-m", commit_message], check=True)
    subprocess.run(["git", "branch", "-M", branch], check=True)
    subprocess.run(["git", "push", "-u", "origin", branch], check=True)

    return f"First commit created and pushed to branch '{branch}' in repository '{name}' at {repo.html_url}"

@app.command("first-commit")
def first_commit_(
    name: str = typer.Argument(..., help="Name of the repository"),
    commit_message: str = typer.Option("Initial commit", "--message", "-m", help="Commit message"),
    branch: str = typer.Option("main", "--branch", "-b", help="Branch name"),
    path: str = typer.Option(".", "--path", "-p", help="Path to the repository")
):
    """
    Create an initial commit in the specified repository.
    """
    result = first_commit(name, commit_message, branch, path)
    typer.echo(result)

@app.command("create")
def create_(
    name: str = typer.Option(..., "-n", "--name", help="Name of the new repository"),
    description: str = typer.Option("", "-d", "--description", help="Description of the repository"),
    private: bool = typer.Option(True, "-p", "--private", help="Set the repository to private"),
    first: bool = typer.Option(False, "--first-commit", "-f", help="Create an initial commit after creating the repository")
):
    """
    Create a new GitHub repository with a description.
    """
    gh = github()
    user = gh.get_user()
    repo = user.create_repo(name=name, description=description, private=private)
    typer.echo(f"Created repository '{repo.full_name}' at {repo.html_url}")

    # Optionally create the first commit
    if first:
        typer.echo("Creating the first commit...")
        result = first_commit(name)
        typer.echo(result)

@app.command(name="new")
def new_():
    """new"""
    typer.echo("Running new subcommand.")


@app.command(name="list")
def list_():
    """
    List all repositories for the authenticated user.
    """
    gh = github()
    user = gh.get_user()
    repos = user.get_repos()

    typer.echo("Your repositories:")
    for repo in repos:
        typer.echo(f"- {repo.full_name}")


@app.command("delete")
def delete_(
    name: str = typer.Option(..., "--name", "-n", help="Name of the repository to delete"),
    confirm: bool = typer.Option(False, "--yes", "-y", help="Skip confirmation prompt")
):
    """
    Delete a GitHub repository.
    """
    gh = github()
    user = gh.get_user()
    
    if not confirm:
        typer.confirm(f"Are you sure you want to delete the repository '{name}'?", abort=True)
    
    user.get_repo(name).delete()
    typer.echo(f"Repository '{name}' has been deleted.")

@app.command("update")
def update_(
    name: str = typer.Option(..., "--name", "-n", help="Name of the repository to update"),
    new_name: str = typer.Option(None, "--new-name", help="New name for the repository"),
    description: str = typer.Option(None, "--description", "-d", help="New description for the repository"),
    private: bool = typer.Option(None, "--private", "-p", help="Set the repository to private")
):
    """
    Update a GitHub repository's properties.
    """
    gh = github()
    user = gh.get_user()
    repo = user.get_repo(name)
    
    if new_name:
        repo.edit(name=new_name)
    if description is not None:
        repo.edit(description=description)
    if private is not None:
        repo.edit(private=private)
    
    typer.echo(f"Repository '{repo.full_name}' has been updated.")

@app.command("clone")
def clone_(
    name: str = typer.Option(..., "--name", "-n", help="Name of the repository to clone"),
    path: str = typer.Option(".", "--path", "-p", help="Path to clone the repository to")
):
    """
    Clone a GitHub repository.
    """
    gh = github()
    user = gh.get_user()
    repo = user.get_repo(name)
    
    import subprocess
    
    clone_command = f"git clone {repo.clone_url} {path}"
    result = subprocess.run(clone_command, shell=True, capture_output=True, text=True)
    
    if result.returncode == 0:
        typer.echo(f"Repository '{repo.full_name}' has been cloned to {path}")
    else:
        typer.echo(f"Error cloning repository: {result.stderr}")

@app.command("fork")
def fork_(
    name: str = typer.Option(..., "--name", "-n", help="Name of the repository to fork"),
    organization: str = typer.Option(None, "--org", "-o", help="Organization to fork the repository to")
):
    """
    Fork a GitHub repository.
    """
    gh = github()
    repo = gh.get_repo(name)
    
    if organization:
        forked_repo = repo.create_fork(organization=organization)
    else:
        forked_repo = repo.create_fork()
    
    typer.echo(f"Repository '{repo.full_name}' has been forked to '{forked_repo.full_name}'")

@app.command("add-collaborator")
def add_collaborator_(
    name: str = typer.Option(..., "--name", "-n", help="Name of the repository"),
    username: str = typer.Option(..., "--user", "-u", help="Username of the collaborator to add"),
    permission: str = typer.Option("push", "--permission", "-p", help="Permission level (pull, push, admin)")
):
    """
    Add a collaborator to a GitHub repository.
    """
    gh = github()
    user = gh.get_user()
    repo = user.get_repo(name)
    
    repo.add_to_collaborators(username, permission=permission)
    typer.echo(f"Added {username} as a collaborator to '{repo.full_name}' with {permission} permissions.")


from confz import BaseConfig, FileSource
from typing import List, Optional
from pydantic import BaseModel


class Chapter(BaseModel):
    title: str
    path: Optional[str] = None
    subchapters: List['Chapter'] = []


class SummaryConfig(BaseConfig):
    title: Optional[str]
    prefix_chapters: List[Chapter] = []
    numbered_chapters: List[Chapter] = []
    suffix_chapters: List[Chapter] = []


def generate_summary_md(config: SummaryConfig) -> str:
    lines = []

    if config.title:
        lines.append(f"# {config.title}\n")

    for chapter in config.prefix_chapters:
        lines.append(f"[{chapter.title}]({chapter.path})\n")

    def process_chapters(chapters, indent=0):
        lines = []
        for chapter in chapters:
            prefix = "    " * indent + "- "
            lines.append(f"{prefix}[{chapter.title}]({chapter.path})\n")
            lines.extend(process_chapters(chapter.subchapters, indent + 1))
        return lines

    lines.extend(process_chapters(config.numbered_chapters))

    for chapter in config.suffix_chapters:
        lines.append(f"[{chapter.title}]({chapter.path})\n")

    return ''.join(lines)


def main():
    """Main function"""
    from sungen.utils.dspy_tools import init_ol
    init_ol()

    config = SummaryConfig()
    print(config)

    summary_md_content = generate_summary_md(config)
    print(summary_md_content)


if __name__ == '__main__':
    main()



import dspy
from sungen.utils.dspy_tools import init_dspy
import pandas as pd
import io

class AskDFSignature(dspy.Signature):
    """
    Answers a natural language question about a DataFrame.
    """
    question = dspy.InputField(desc="Natural language question about the DataFrame.")
    df_csv = dspy.InputField(desc="The DataFrame in CSV string format.")
    answer = dspy.OutputField(desc="Plain text answer to the question.")

class AskDFModule(dspy.Module):
    """AskDFModule for answering questions about DataFrames using natural language"""

    def __init__(self, **forward_args):
        super().__init__()
        self.forward_args = forward_args

    def forward(self, question, df):
        # Convert DataFrame to CSV string
        csv_buffer = io.StringIO()
        df.to_csv(csv_buffer, index=False)
        df_csv = csv_buffer.getvalue()

        pred = dspy.Predict(AskDFSignature)
        return pred(question=question, df_csv=df_csv).answer

def ask_df_call(question, df):
    ask_df_module = AskDFModule()
    return ask_df_module.forward(question=question, df=df)

def main():
    init_dspy()
    # Example usage
    df = pd.DataFrame({
        'name': ['Alice', 'Bob', 'Charlie'],
        'age': [25, 30, 35],
        'city': ['New York', 'San Francisco', 'London']
    })
    question = "Who is older than 30?"
    
    result = ask_df_call(question=question, df=df)
    print(result)

if __name__ == "__main__":
    main()


from rich.console import Console
from rich.panel import Panel
from rich.text import Text

console = Console()

def echo(message: str, style: str = "default", panel: bool = False, title: str = None):
    """
    Custom echo function with rich formatting.
    
    Args:
    message (str): The message to be printed.
    style (str): The style to apply to the message (e.g., "bold", "italic", "green").
    panel (bool): Whether to wrap the message in a panel.
    title (str): The title of the panel (only used if panel is True).
    """
    if panel:
        console.print(Panel(Text(message, style=style), title=title))
    else:
        console.print(Text(message, style=style))


import inspect
from typing import Any, Set, get_args, get_origin, List, Union, Literal, Optional
from enum import Enum

from dslmodel import DSLModel


def collect_class_sources(cls: Any, collected_sources: Set[str]) -> None:
    """
    Recursively collect source code for the given class and all related classes
    (based on fields' type annotations) that inherit from DSLModel, including Enums.
    """
    # Skip if we've already processed this class by its name
    if cls.__name__ in collected_sources:
        return

    # Collect the source code for the class itself
    try:
        source_code = inspect.getsource(cls)
        collected_sources.add(source_code)  # Add the source code as a string, not the class object
    except (TypeError, OSError) as e:
        print(f"Unable to retrieve source for {cls.__name__}: {str(e)}")

    # Now process the fields of the class
    if hasattr(cls, 'model_fields'):  # Pydantic v2
        fields = cls.model_fields
    elif hasattr(cls, '__fields__'):  # Pydantic v1
        fields = cls.__fields__
    else:
        print(f"{cls.__name__} does not appear to be a Pydantic model.")
        return

    # Recursively check if any field annotations are also DSLModel classes
    for field_name, field in fields.items():
        field_type = field.annotation

        # Handle cases where the field is a complex type (List, Optional, Union, Enum, etc.)
        handle_complex_type(field_type, collected_sources)


def handle_complex_type(field_type: Any, collected_sources: Set[str]) -> None:
    """
    Handles complex types like List, Union, Optional, Literal, Enum, etc., and recursively collects
    source code for any referenced classes that inherit from DSLModel or are Enums.
    """
    origin_type = get_origin(field_type)  # Get the origin (e.g., List, Union, Literal)
    args = get_args(field_type)  # Get the inner types (e.g., for Union or List)

    # If it's a List, Union, or Optional, process its inner types recursively
    if origin_type in (list, List, Union, Optional):
        for arg in args:
            if arg is not type(None):  # Skip 'NoneType' from Optional or Union
                if hasattr(arg, '__bases__'):
                    if issubclass(arg, DSLModel):
                        collect_class_sources(arg, collected_sources)
                    elif issubclass(arg, Enum):
                        collect_enum_sources(arg, collected_sources)

    # If it's a Literal, we don't need to process further
    elif origin_type is Literal:
        pass

    # If it's directly a DSLModel or Enum, collect its source
    elif hasattr(field_type, '__bases__'):
        if issubclass(field_type, DSLModel):
            collect_class_sources(field_type, collected_sources)
        elif issubclass(field_type, Enum):
            collect_enum_sources(field_type, collected_sources)

def collect_enum_sources(enum_cls: Enum, collected_sources: Set[str]) -> None:
    """
    Collects the source code for an Enum class.
    """
    # Skip if we've already processed this enum by its name
    if enum_cls.__name__ in collected_sources:
        return

    try:
        source_code = inspect.getsource(enum_cls)
        collected_sources.add(source_code)
    except (TypeError, OSError) as e:
        print(f"Unable to retrieve source for {enum_cls.__name__}: {str(e)}")

def collect_all_sources_as_string(cls: Any) -> str:
    """
    Collects the source code for the class and all related DSLModel classes, including Enums,
    and returns them as a single string.
    """
    collected_sources = set()

    # Collect sources starting from the given class
    collect_class_sources(cls, collected_sources)

    # Join all collected source codes into a single string
    return "\n\n".join(collected_sources)



def main():
    """Main function"""
    from sungen.utils.dspy_tools import init_lm
    init_lm()

    # Now, let's collect the sources from the GherkinDocument class:
    all_sources_string = collect_all_sources_as_string(GherkinDocument)

    # Print the result
    print(all_sources_string)


if __name__ == '__main__':
    main()



import json

import jsonschema
from jsonschema.validators import validate


def extract(string) -> dict or None:
    bracket_pairs = {
        '{': '}',
        '[': ']',
    }

    # Find the index of the first opening bracket
    start_index = -1
    for i, char in enumerate(string):
        if char in bracket_pairs:
            start_index = i
            break

    if start_index == -1:
        return None

    opening_char = string[start_index]
    closing_char = bracket_pairs[opening_char]

    count = 0
    end_index = -1

    # Slice the string from the found opening bracket
    substring = string[start_index:]

    # Iterate over the substring to find the matching closing bracket
    for i, char in enumerate(substring):
        if char == opening_char:
            count += 1
        elif char == closing_char:
            count -= 1

        if count == 0:
            end_index = i
            break

    if end_index == -1:
        return None

    # Parse the JSON-like substring into a Python object
    try:
        return json.loads(substring[:end_index + 1])
    except json.JSONDecodeError:
        return None


from faker import Faker
import json
import random

fake = Faker()


def generate_from_schema(schema, check_validity=False, strip_properties=True):
    if type(schema) is str:
        schema = json.loads(schema)
    instance = generate_instance(schema)
    # Validate the instance against the schema
    if check_validity:
        validate_instance(instance, schema)
    if strip_properties:
        instance = strip_non_validation_properties(instance)
    return instance


def generate_instance(schema):
    schema_type = schema.get('type', 'string')
    if schema_type == 'object':
        return {k: generate_instance(v) for k, v in schema.get('properties', {}).items()}
    elif schema_type == 'array':
        item_schema = schema.get('items', {})
        # Assume a default size of the array if not specified
        length = schema.get('minItems', 1)
        return [generate_instance(item_schema) for _ in range(length)]
    elif schema_type == 'string':
        return generate_string(schema)
    elif schema_type == 'number':
        return fake.pyfloat(left_digits=None, right_digits=None, positive=True)
    elif schema_type == 'integer':
        return fake.pyint()
    elif schema_type == 'boolean':
        return fake.pybool()
    else:
        return generate_default(schema)


def generate_string(schema):
    format_type = schema.get('format')
    pattern = schema.get('pattern')

    if format_type == 'email':
        return fake.email()
    elif format_type == 'date':
        return fake.date()
    elif format_type == 'date-time':
        return fake.date_time().isoformat()
    elif format_type == 'uri':
        return fake.uri()
    elif pattern:
        # Simplistic pattern handling for specific cases (demonstration purposes)
        if pattern == '^[0-1]$':
            return str(random.choice(['0', '1']))
        elif pattern == '^[0-2]$':
            return str(random.choice(['0', '1', '2']))
        else:
            # Generic handler for simple patterns
            return generate_matching_string(pattern)
    else:
        return fake.word()


def generate_matching_string(pattern):
    # This is a placeholder function, needs a real implementation
    # For now, it will just return a fixed correct example for known patterns:
    if pattern == '^[0-1]$':
        return random.choice(['0', '1'])
    elif pattern == '^[0-2]$':
        return random.choice(['0', '1', '2'])
    # You would need to extend this to handle other patterns or write a general regex string generator
    return "0"


def validate_instance(instance, schema):
    try:
        validate(instance=instance, schema=schema)
    except jsonschema.exceptions.ValidationError as e:
        raise ValueError(f"Generated instance does not comply with the schema: {e}")


def generate_default(schema):
    if 'default' in schema:
        return schema['default']
    return None


def strip_non_validation_properties(schema):
    # Define the list of non-validation properties to remove
    non_validation_keys = {"title", "description", "examples"}

    # If the schema is a dictionary, iterate through its items
    if isinstance(schema, dict):
        keys_to_remove = [key for key in schema if key in non_validation_keys]
        for key in keys_to_remove:
            del schema[key]  # Remove the non-validation properties

        # Recursively process nested dictionaries or arrays
        for key, value in schema.items():
            if isinstance(value, dict) or isinstance(value, list):
                strip_non_validation_properties(value)

    # If the schema is a list, process each element in the list
    elif isinstance(schema, list):
        for item in schema:
            strip_non_validation_properties(item)

    return schema


def main2():
    # Example schema with non-validation properties
    json_schema_example = {
        "title": "User",
        "type": "object",
        "properties": {
            "name": {
                "title": "Name",
                "type": "string",
                "description": "The person's name"
            },
            "age": {
                "title": "Age",
                "type": "integer",
                "description": "The person's age"
            }
        },
        "required": ["name", "age"],
        "description": "A simple user schema"
    }

    # Use the function to strip non-validation properties
    cleaned_schema = strip_non_validation_properties(json_schema_example)
    print(cleaned_schema)


def main():
    """Main function"""
    from sungen.utils.dspy_tools import init_ol
    init_ol()

    # Example JSON schema with nested objects and arrays
    schema = {
        "type": "object",
        "properties": {
            "user": {
                "type": "object",
                "properties": {
                    "name": {"type": "string"},
                    "email": {"type": "string", "format": "email"},
                    "tags": {
                        "type": "array",
                        "items": {"type": "string"}
                    }
                }
            },
            "logins": {
                "type": "array",
                "items": {
                    "type": "object",
                    "properties": {
                        "date": {"type": "string", "format": "date-time"},
                        "successful": {"type": "boolean"}
                    }
                }
            }
        }
    }

    # Generate an instance that matches the schema
    print(json.dumps(generate_from_schema(schema), indent=4))


def main3():
    import json
    from jsonschema import validate, Draft202012Validator, exceptions

    schema = {"$id": "https://example.com/fstab", "$schema": "https://json-schema.org/draft/2020-12/schema", "type": "object", "required": ["/"], "properties": {"/": {"type": "object", "properties": {"device": {"type": "string"}, "mount_point": {"type": "string"}, "file_system_type": {"type": "string"}, "options": {"type": "string"}, "dump": {"type": "string", "pattern": "^[0-1]$"}, "pass": {"type": "string", "pattern": "^[0-2]$"}}, "required": ["device", "mount_point", "file_system_type", "options", "dump", "pass"]}}, "patternProperties": {"^(/[^/]+)+$": {"type": "object", "properties": {"device": {"type": "string"}, "mount_point": {"type": "string"}, "file_system_type": {"type": "string"}, "options": {"type": "string"}, "dump": {"type": "string", "pattern": "^[0-1]$"}, "pass": {"type": "string", "pattern": "^[0-2]$"}}, "required": ["device", "mount_point", "file_system_type", "options", "dump", "pass"]}}, "additionalProperties": False}
    # Test instance
    instance = {
        "/": {
            "device": "/dev/sda1",
            "mount_point": "/",
            "file_system_type": "ext4",
            "options": "defaults",
            "dump": "1",
            "pass": "2"
        },
        "/home": {
            "device": "/dev/sda2",
            "mount_point": "/home",
            "file_system_type": "ext4",
            "options": "defaults",
            "dump": "0",
            "pass": "1"
        }
    }

    try:
        Draft202012Validator(schema).validate(instance)
        print("The instance is valid!")
    except exceptions.ValidationError as e:
        print(f"Validation error: {e.message}")


if __name__ == '__main__':
    # main()
    main3()



import dspy
import pyperclip
from typer import Typer
from inflection import underscore, dasherize
from datetime import datetime

from sungen.utils.str_tools import pythonic_str
from zipp import Path

app = Typer(
    help="Generate a file name from any text, with optional timestamp and safe file format handling."
)

# Define safe time formats for filenames (no invalid characters)
class TimeFormats:
    ISO_LIKE = "%Y-%m-%dT%H%M%S"                    # 2024-09-27T153000 (ISO-like without colons)
    YEAR_MONTH_DAY_UNDERSCORE = "%Y-%m-%d"          # 2024-09-27
    FULL_DATETIME_UNDERSCORE = "%Y-%m-%d-%H%M%S"    # 2024-09-27-153000
    YEAR_MONTH_DAY_COMPACT = "%Y%m%d"               # 20240927 (Compact date only)
    DATE_HOUR_MIN_SEC_COMPACT = "%Y%m%d-%H%M%S"     # 20240927-153000
    TIME_ONLY = "%H%M%S"                            # 153000 (Time only)
    HUMAN_READABLE_UNDERSCORE = "%B-%d-%Y"          # September-27-2024 (Human-readable, safe)
    YEAR_MONTH = "%Y-%m"                            # 2024-09 (Year and month only)


class WindowsSafeFileName(dspy.Signature):
    """
    Generate a Windows-safe filename from file content without adding the file extension.
    Ensures the filename complies with Windows naming conventions and restrictions.
    """

    file_content = dspy.InputField(desc="Text content that needs to be converted to a valid Windows-safe filename.")
    safe_filename = dspy.OutputField(desc="The generated Windows-safe filename.", prefix="filename:")


class FileContentToFileNameModule(dspy.Module):
    """Converts file content to a safe file name with an optional timestamp and extension."""

    def __init__(self, extension: str = None, time_format: str = None, add_timestamp: bool = False):
        super().__init__()
        self.extension = extension
        self.time_format = time_format
        self.add_timestamp = add_timestamp

    def forward(self, file_content: str) -> str:
        """
        Converts the provided file content to a valid filename with the specified extension.
        Optionally appends a timestamp using a specified time format.
        """
        # Chain of Thought to generate a valid file name
        pred = dspy.ChainOfThought(WindowsSafeFileName)

        # Generate the file name from the content
        result = pred(file_content=file_content).safe_filename
        result = result.replace(" ", "-")

        # Convert to snake_case if the extension is .py
        if self.extension == "py":
            result = pythonic_str(result)

        # Add a timestamp if required
        if self.add_timestamp and self.time_format:
            current_time = datetime.now().strftime(self.time_format)
            result = f"{result}_{current_time}"

        # Append the extension to the file name if provided
        if self.extension:
            result = f"{result}.{self.extension}"

        return result


def file_name_call(file_content: str, extension: str = None, time_format: str = TimeFormats.YEAR_MONTH_DAY_UNDERSCORE, add_timestamp: bool = False) -> str:
    """Generates the file name from content with an optional timestamp and file extension."""
    file_content_to_file_name = FileContentToFileNameModule(
        extension=extension, time_format=time_format, add_timestamp=add_timestamp
    )
    return file_content_to_file_name.forward(file_content=file_content)


def main():
    # Get file content from clipboard (or initialize with other input)
    from dspygen.utils.dspy_tools import init_versatile
    init_versatile()
    file_content = pyperclip.paste()

    # Example usage of file_name_call with a timestamp
    file_name = file_name_call(
        file_content=file_content,
        extension="md",  # Example: Python file extension
        time_format=TimeFormats.FULL_DATETIME_UNDERSCORE,  # Example safe timestamp format
        add_timestamp=True
    ) 

    print(file_name)

    # write the file to my obsidian vault
    file_path = "/Users/sac/dev/vault/myvault/" + file_name
    with open(file_path, "w") as file:
        file.write(file_content)

@app.command()
def call(file_content: str, extension: str = None, add_timestamp: bool = False, time_format: str = None):
    """CLI command to convert file content to a file name with optional timestamp."""
    file_name = file_name_call(
        file_content=file_content,
        extension=extension,
        time_format=time_format if time_format else TimeFormats.YEAR_MONTH_DAY_UNDERSCORE,
        add_timestamp=add_timestamp
    )
    print(file_name)



def watch_clipboard():
    """Watch the clipboard for changes and call the main function when it changes."""
    clipboard_content = pyperclip.paste()

    while True:
        new_clipboard_content = pyperclip.paste()
        if new_clipboard_content != clipboard_content:
            import time
            time.sleep(0.01)  # Sleep for 0.01 seconds to ensure clipboard content is fully updated
            clipboard_content = new_clipboard_content
            main()
    

if __name__ == "__main__":
    # For running via CLI
    watch_clipboard()  # For direct execution
    # Uncomment below to use the Typer CLI:
    # app()



"""
bdd Plugin CLI
This file contains only the CLI commands for the auto_bdd plugin.
"""
import dspy
import typer
from pathlib import Path
from typing import Optional
import importlib.util
import subprocess
import sys

from sungen.lm.cerebras import Cerebras
from sungen.utils.dspy_tools import init_dspy

app = typer.Typer(name="bdd")


def check_installation():
    """
    Check if pytest-bdd is installed and install it if not present.
    Also check for pytest and warn if it is missing.
    """
    try:
        if importlib.util.find_spec("pytest") is None:
            print("pytest is not installed. Installing...")
            subprocess.check_call([sys.executable, "-m", "pip", "install", "pytest"])
            print("pytest has been installed successfully.")

        if importlib.util.find_spec("pytest_bdd") is None:
            print("pytest-bdd is not installed. Installing...")
            subprocess.check_call([sys.executable, "-m", "pip", "install", "pytest-bdd"])
            print("pytest-bdd has been installed successfully.")
    except Exception as e:
        print(f"Error during installation check: {str(e)}")


def generate_gherkin(prompt: str, file_path: Path) -> None:
    """
    Generate a Gherkin feature file based on a prompt.
    """
    # Initialize DSPy model
    init_dspy(lm_class=Cerebras, max_tokens=2000)

    # Use DSPy Predictor to generate Gherkin text
    class GherkinInfo(dspy.Signature):
        prompt = dspy.InputField(desc="Text prompt containing details or requirements for the Gherkin feature.")
        file_name = dspy.OutputField(desc="{{feature_file_name}}.feature (no other text)")
        gherkin_text = dspy.OutputField(desc="Generated Gherkin feature file content.")

    predictor = dspy.Predict(GherkinInfo)
    output = predictor(prompt=prompt)

    print(f"Generated Gherkin file: {output.file_name}")

    # LLM sometimes returns features/File Name: login.feature, so split on " " and take the last element
    feature_file_name = output.file_name.split(" ")[-1]

    # Ensure the tests directory exists
    tests_directory = tests_dir()
    tests_directory.mkdir(parents=True, exist_ok=True)

    if not file_path.exists():
        Path(tests_directory / feature_file_name).write_text(output.gherkin_text)
    else:
        file_path.write_text(output.gherkin_text)


def run_code_generator(feature_file: Path, output_dir: Path) -> None:
    """
    Run the test code generator based on the provided Gherkin feature file.
    """
    # Ensure pytest-bdd and pytest are available
    # Removed import check here
    typer.echo(f"Running code generator for feature file: {feature_file}")
    command = f"pytest-bdd generate {feature_file}"
    result = subprocess.run(command, shell=True, capture_output=True, text=True)

    # Check for errors in the code generation process
    if result.returncode != 0:
        raise RuntimeError(f"Error running code generator: {result.stderr}")

    # Save generated code to the output directory
    output_path = output_dir / f"test_{feature_file.stem}.py"
    with open(output_path, 'w') as test_file:
        test_file.write(result.stdout)

    typer.echo(f"Generated test code saved to {output_path}")


@app.command(name="create")
def generate_gherkin_command(
        prompt: str = typer.Argument(..., help="Prompt describing the feature or scenario to generate in Gherkin."),
        output_file: Path = typer.Option(Path("features/generated.feature"),
                                         help="Output file for the generated Gherkin feature."),
        output_dir: Optional[Path] = typer.Option(Path("features"),
                                                  help="Output directory for generated Gherkin feature."),
        generate_code: bool = typer.Option(False, help="Generate code for the feature file.")
):
    """
    CLI Command: Generate a Gherkin feature file based on a prompt.

    Usage:
        sungen bdd create "As a user, I want to log in" --output-file features/login.feature --generate-code
    """
    # Only validate input, no business logic here
    if not prompt:
        typer.echo("Prompt is required.")
        raise typer.Exit(code=1)

    try:
        generate_gherkin(prompt, output_file)

        if generate_code:
            run_code_generator(output_file, output_dir)
    except Exception as e:
        typer.echo(f"Failed to generate Gherkin feature: {str(e)}")
        raise typer.Exit(code=1)


@app.command(name="gen")
def run_code_generator_command(
        feature_file: Path = typer.Argument(..., help="Path to the Gherkin feature file."),
        output_dir: Optional[Path] = typer.Option(Path("tests/functional"),
                                                  help="Output directory for generated test code.")
):
    """
    CLI Command: Run the test code generator based on the provided Gherkin feature file.

    Usage:
        sungen bdd gen features/login.feature --output-dir tests/functional
    """
    # Only validate input, no business logic here
    if not feature_file.exists():
        typer.echo("Feature file not found.")
        raise typer.Exit(code=1)

    try:
        run_code_generator(feature_file, output_dir)
    except Exception as e:
        typer.echo(f"Failed to run code generator: {str(e)}")
        raise typer.Exit(code=1)


def register_plugin(parent_app: typer.Typer):
    """
    Register the plugin with the main application.
    """
    try:
        check_installation()
        parent_app.add_typer(app)

        if app in parent_app.registered_groups:
            raise Exception("Failed to register plugin 'bdd'")
    except Exception as e:
        print(f"Failed to register plugin 'bdd': {str(e)}")


if __name__ == "__main__":
    app()



import subprocess
import typer
from typing import Optional, List
import os
import importlib.util
import sys

app = typer.Typer(help="Sungen Aider Plugin for AI-assisted coding tasks", name="aider")


def run_aider_command(args: List[str], stream: bool = True):
    """
    Run an Aider command with the provided arguments.

    Args:
        args (list): List of arguments for the Aider CLI command.
        stream (bool): Enable/disable streaming responses.
    """
    try:
        cmd = ['aider'] + args
        if not stream:
            cmd.append('--no-stream')
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        print(result.stdout)
    except subprocess.CalledProcessError as e:
        print(f"Error running Aider command: {e.stderr}")


@app.command()
def message(model: str, prompt: str, files: Optional[List[str]] = typer.Argument(None)):
    """Send a message to Aider to perform a specific task on specified files."""
    args = ['--model', model, '--message', prompt]
    if files:
        args.extend(files)
    run_aider_command(args)


@app.command()
def generate(language: str, description: str, output_file: str):
    """Generate code or scripts in the specified language and save to a file."""
    prompt = f"Generate {language} code: {description}"
    run_aider_command(['--message', prompt, '--file', output_file])


@app.command()
def refactor(file: str, changes: str):
    """Refactor or modify the code in the given file."""
    prompt = f"Refactor the code in {file}: {changes}"
    run_aider_command(['--message', prompt, file])


@app.command()
def lint(file: Optional[str] = None, auto_fix: bool = False):
    """Perform linting checks on the specified file or all files."""
    args = ['--lint']
    if auto_fix:
        args.append('--auto-lint')
    if file:
        args.extend(['--file', file])
    run_aider_command(args)


@app.command()
def setup(model: str, api_key: str):
    """Setup Aider with the specified model and API key."""
    key_var = 'OPENAI_API_KEY' if 'gpt' in model else 'ANTHROPIC_API_KEY'
    os.environ[key_var] = api_key
    print(f"Aider setup complete with model {model}.")


@app.command()
def interactive(model: str = "gpt-4o-mini", files: Optional[List[str]] = typer.Argument(None)):
    """Enter interactive mode with Aider using the specified model and optional files."""
    args = ['--model', model]
    if files:
        args.extend(files)
    run_aider_command(args)


@app.command()
def commit(message: Optional[str] = None):
    """Commit all pending changes with an optional commit message."""
    args = ['--commit']
    if message:
        args.extend(['--commit-prompt', message])
    run_aider_command(args)


def check_installation():
    """
    Check if aider-chat is installed and install it if not present.
    Also check for API keys and warn if both are missing.
    """
    try:
        if importlib.util.find_spec("aider") is None:
            print("aider-chat is not installed. Installing...")
            subprocess.check_call([sys.executable, "-m", "pip", "install", "aider-chat"])
            print("aider-chat has been installed successfully.")

        openai_key = os.environ.get('OPENAI_API_KEY')
        anthropic_key = os.environ.get('ANTHROPIC_API_KEY')

        if not openai_key and not anthropic_key:
            print("Warning: Both OPENAI_API_KEY and ANTHROPIC_API_KEY are missing. At least one is recommended for Aider.")
    except Exception as e:
        print(f"Error during installation check: {str(e)}")


def register_plugin(parent_app: typer.Typer):
    """
    Register the plugin with the main application.
    """
    try:
        check_installation()
        parent_app.add_typer(app)
    except Exception as e:
        print(f"Failed to register plugin '{app}': {str(e)}")


if __name__ == "__main__":
    app()



import importlib

import dspy

from sungen.dsl.dsl_pydantic_models import SignatureDSLModel, GenSignatureModel
from sungen.utils.file_tools import dsl_dir


def get_sig_key(module_def, step):
    sig_key = ""

    if module_def and module_def.signature != "":
        sig_key = module_def.signature

    if step.signature != "":
        sig_key = step.signature

    return sig_key


def _process_module_signatures(global_signatures, module_def, step):
    """
    Process the signatures for a module definition and a step. Load the signature class if not already loaded.
    """
    sig_key = get_sig_key(module_def, step)

    if sig_key in global_signatures:
        return

    sig_cls = _load_signature_class(sig_key)

    global_signatures[sig_key] = sig_cls


def _create_signature_from_model(signature_model: SignatureDSLModel) -> type:
    """
    Create a DSPy Signature class from a SignatureDSLModel instance.
    """
    class_dict = {"__doc__": signature_model.docstring, "__annotations__": {}}

    # Process input fields
    for input_field in signature_model.inputs:
        name = input_field.name
        desc = input_field.desc
        field_instance = dspy.InputField(desc=desc)
        class_dict[name] = field_instance
        class_dict["__annotations__"][name] = dspy.InputField

    # Process output fields
    for output_field in signature_model.outputs:
        name = output_field.name
        desc = output_field.desc
        prefix = output_field.prefix
        field_instance = dspy.OutputField(prefix=prefix, desc=desc)
        class_dict[name] = field_instance
        class_dict["__annotations__"][name] = dspy.OutputField

    # Dynamically create the Signature class
    signature_class = type(signature_model.name, (dspy.Signature,), class_dict)
    return signature_class


def _load_signature_class(signature_class_name: str):
    """
    Dynamically loads a signature class by its fully qualified name.
    """
    if signature_class_name.endswith(".yaml") or signature_class_name.endswith(".yml"):
        if signature_class_name.startswith("/"):
            signature_model = GenSignatureModel.from_yaml(signature_class_name)
        else:
            signature_model = GenSignatureModel.from_yaml(str(dsl_dir(signature_class_name)))
        return _create_signature_from_model(signature_model)
    else:
        module_name, class_name = signature_class_name.rsplit('.', 1)
        module = importlib.import_module(module_name)
        return getattr(module, class_name)




def main():
    """Main function"""
    from sungen.utils.dspy_tools import init_ol
    init_ol()

    import requests

    # Define the API endpoint
    url = 'https://api.cerebras.ai/v1/chat/completions'

    # Set the headers
    headers = {
        'accept': 'application/json',
        'accept-language': 'en-US,en;q=0.9',
        'authorization': 'Bearer demo-xmyc249jrym6xx6r5ty3k4cfkn6rpvyj48dj5k3hffcf2pp5',
        'content-type': 'application/json',
        'dnt': '1',
        'origin': 'https://inference.cerebras.ai',
        'priority': 'u=1, i',
        'referer': 'https://inference.cerebras.ai/',
        'sec-ch-ua': '"Chromium";v="128", "Not;A=Brand";v="24", "Google Chrome";v="128"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"macOS"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-site',
        'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/128.0.0.0 Safari/537.36',
        'x-stainless-arch': 'unknown',
        'x-stainless-lang': 'js',
        'x-stainless-os': 'Unknown',
        'x-stainless-package-version': '1.0.1',
        'x-stainless-runtime': 'browser:chrome',
        'x-stainless-runtime-version': '128.0.0'
    }

    # Define the payload with only the first 2 messages
    data = {
        "messages": [
            {
                "content": "Please try to provide useful, helpful and actionable answers.",
                "role": "system"
            },
            {
                "content": "Ridiculously fast LLM inference would unlock...",
                "role": "user"
            }
        ],
        "model": "llama3.1-8b",
        "stream": False,
        "temperature": 0.2,
        "top_p": 1,
        "max_tokens": 4096
    }

    # Send the POST request
    response = requests.post(url, headers=headers, json=data)

    # Print the response from the server
    print(response.json())


if __name__ == '__main__':
    main()



"""assist"""
import typer
import os
import tempfile
import subprocess
from sungen.plugins.blueprint.models.blueprint_models import CodeBlueprint  # Import the model
from pathlib import Path

app = typer.Typer()


@app.command(name="blueprint")
def _blueprint(echo: str):
    """blueprint"""
    typer.echo(f"Running blueprint subcommand: {echo}")


@app.command(name="run")
def run(file_path: Path = Path("./blueprint.yaml")):
    """run"""
    _run(file_path)


def _run(file_path: Path):
    """Run a specified blueprint from the given file path."""
    bp = CodeBlueprint.from_yaml(file_path)
    print(bp)

    for file in bp.files_to_create:
        directory = os.path.dirname(file)
        if directory and not os.path.exists(directory):
            typer.echo(f"Creating directory: {directory}")
            os.makedirs(directory)  # Create the missing directories
        if not os.path.exists(file):
            typer.echo(f"Creating file: {file}")
            with open(file, 'w') as f:
                f.write('')  # Create an empty file

    # Add logic to execute the blueprint here
    # For example, you might want to load the blueprint from the file path
    # blueprint = load_blueprint(file_path)
    # process_blueprint(blueprint)

def run_blueprint_logic(blueprint_file: str):
    """Logic for running a specified blueprint."""
    # Load the blueprint from the YAML file
    blueprint = CodeBlueprint.from_yaml(blueprint_file)

    # Step 1: Handle `files_to_create` - Create any files that are listed
    for file_to_create in blueprint.files_to_create:
        directory = os.path.dirname(file_to_create)
        if directory and not os.path.exists(directory):
            typer.echo(f"Creating directory: {directory}")
            os.makedirs(directory)  # Create the missing directories
        if not os.path.exists(file_to_create):
            typer.echo(f"Creating file: {file_to_create}")
            with open(file_to_create, 'w') as f:
                f.write('')  # Create an empty file for other types

    # Step 2: Construct the base aider command
    command = [
        "aider",
        "--model", blueprint.model,  # Use the specified AI model
        "--yes",  # Assume yes to all prompts for smoother automation
        # "--verbose",
        # "--no-stream"
    ]

    for file in (blueprint.files_to_edit + blueprint.files_to_create):
        command.extend(["--file", file])  # Specify files to change

    for read_only_file in blueprint.read_only_files:
        command.extend(["--read", read_only_file])  # Specify files as read-only

    if blueprint.test_cmd:
        command.append("--auto-test")
        command.append(f"--test-cmd={blueprint.test_cmd}")
    if blueprint.lint:
        command.append("--lint")
    if not blueprint.auto_commit:
        command.append("--no-auto-commits")
    if blueprint.verbose:
        command.append("--verbose")

    # Step 6: Handle `context_files` - Add any context files as read-only files
    for context_file in blueprint.context_files:
        command.extend(["--read", context_file])

    # Step 7: Handle `additional_args` - Append any extra arguments
    command.extend(blueprint.additional_args)

    # Step 8: Handle `message` - Write the message to a temp file if provided
    if blueprint.message:
        with tempfile.NamedTemporaryFile(delete=False, mode='w') as temp_message_file:
            temp_message_file.write(blueprint.message)
            temp_message_file_path = temp_message_file.name
        command.append(f"--message-file={temp_message_file_path}")  # Use the temp file path

    # Print and execute the command
    typer.echo(f"Running command: {' '.join(command)}")
    subprocess.run(command)
    
    # Step 9: Clean up the temp message file if it exists
    if os.path.exists(temp_message_file_path):
        os.remove(temp_message_file_path)


import typer

from sungen.utils.git_tools import github

# Create a Typer app for the branch commands
app = typer.Typer()

# Command to create a new branch
@app.command("create")
def create_branch(
    repo_name: str = typer.Option(..., "-r", "--repo", help="Name of the repository"),
    branch_name: str = typer.Option(..., "-b", "--branch", help="Name of the new branch"),
    base_branch: str = typer.Option("main", "-B", "--base", help="Base branch to create the new branch from"),
):
    repo = github().get_user().get_repo(repo_name)
    base_ref = repo.get_git_ref(f"heads/{base_branch}")
    new_ref = repo.create_git_ref(ref=f"refs/heads/{branch_name}", sha=base_ref.object.sha)

    typer.echo(f"Created new branch '{branch_name}' from '{base_branch}' in repository '{repo_name}'.")

# Command to delete a branch
@app.command("delete")
def delete_branch(
    repo_name: str = typer.Option(..., "-r", "--repo", help="Name of the repository"),
    branch_name: str = typer.Option(..., "-b", "--branch", help="Name of the branch to delete"),
):
    repo = github().get_user().get_repo(repo_name)
    ref = repo.get_git_ref(f"heads/{branch_name}")
    ref.delete()

    typer.echo(f"Deleted branch '{branch_name}' from repository '{repo_name}'.")

# Command to list all branches in a repository
@app.command("list")
def list_branches(
    repo_name: str = typer.Option(..., "-r", "--repo", help="Name of the repository"),
):
    repo = github().get_user().get_repo(repo_name)
    branches = repo.get_branches()

    typer.echo(f"Branches in repository '{repo_name}':")
    for branch in branches:
        typer.echo(f"- {branch.name}")

@app.command("merge")
def merge_branch(
    repo_name: str = typer.Option(..., "-r", "--repo", help="Name of the repository"),
    head: str = typer.Option(..., "--head", help="The name of the branch to merge from"),
    base: str = typer.Option(..., "--base", help="The name of the branch to merge into"),
    commit_message: str = typer.Option(None, "--message", "-m", help="Commit message for the merge")
):
    """Merge one branch into another."""
    repo = github().get_user().get_repo(repo_name)
    merge_result = repo.merge(base, head, commit_message)
    typer.echo(f"Merged '{head}' into '{base}'. Commit SHA: {merge_result.sha}")

@app.command("protect")
def protect_branch(
    repo_name: str = typer.Option(..., "-r", "--repo", help="Name of the repository"),
    branch_name: str = typer.Option(..., "-b", "--branch", help="Name of the branch to protect"),
    require_pr: bool = typer.Option(True, help="Require pull request reviews before merging"),
    required_approvals: int = typer.Option(1, help="Number of required approving reviews")
):
    """Set branch protection rules."""
    repo = github().get_user().get_repo(repo_name)
    branch = repo.get_branch(branch_name)
    branch.edit_protection(required_approving_review_count=required_approvals, require_pull_request_reviews=require_pr)
    typer.echo(f"Protection rules set for branch '{branch_name}' in repository '{repo_name}'.")

@app.command("rename")
def rename_branch(
    repo_name: str = typer.Option(..., "-r", "--repo", help="Name of the repository"),
    old_name: str = typer.Option(..., "--old", help="Current name of the branch"),
    new_name: str = typer.Option(..., "--new", help="New name for the branch")
):
    """Rename a branch."""
    repo = github().get_user().get_repo(repo_name)
    git_ref = repo.get_git_ref(f"heads/{old_name}")
    git_ref.edit(ref=f"refs/heads/{new_name}")
    repo.get_git_ref(f"heads/{old_name}").delete()
    typer.echo(f"Branch '{old_name}' renamed to '{new_name}' in repository '{repo_name}'.")

@app.command("compare")
def compare_branches(
    repo_name: str = typer.Option(..., "-r", "--repo", help="Name of the repository"),
    base: str = typer.Option(..., "--base", help="Base branch for comparison"),
    head: str = typer.Option(..., "--head", help="Head branch for comparison")
):
    """Compare two branches."""
    repo = github().get_user().get_repo(repo_name)
    comparison = repo.compare(base, head)
    typer.echo(f"Comparing {base}...{head} in repository '{repo_name}':")
    typer.echo(f"Commits ahead: {comparison.ahead_by}")
    typer.echo(f"Commits behind: {comparison.behind_by}")
    typer.echo(f"Diff URL: {comparison.html_url}")

@app.command("set-default")
def set_default_branch(
    repo_name: str = typer.Option(..., "-r", "--repo", help="Name of the repository"),
    branch_name: str = typer.Option(..., "-b", "--branch", help="Name of the branch to set as default")
):
    """Set the default branch for a repository."""
    repo = github().get_user().get_repo(repo_name)
    repo.edit(default_branch=branch_name)
    typer.echo(f"Default branch for repository '{repo_name}' set to '{branch_name}'.")

"""
mix archive.install hex phx_new

mix igniter.new helpdesk \
  --install ash,ash_postgres \
  --with phx.new \
  --extend postgres \
  --example
"""


from jinja2 import Environment, FileSystemLoader

from sungen.typetemp.extension.faker_extension import FakerExtension
from sungen.typetemp.extension.inflection_extension import InflectionExtension
from sungen.utils.file_tools import templates_dir


class TypedEnvironment(Environment):
    def __init__(self, **kwargs):
        super(TypedEnvironment, self).__init__(
            trim_blocks=True, lstrip_blocks=True, **kwargs
        )

        self.add_extension(FakerExtension)
        self.add_extension(InflectionExtension)
        self.add_extension("jinja2_time.TimeExtension")
        self.add_extension("jinja2.ext.i18n")
        self.add_extension("jinja2.ext.debug")
        self.add_extension("jinja2.ext.do")
        self.add_extension("jinja2.ext.loopcontrols")

        self.filters["to_kwarg"] = lambda input_name: f"{input_name}={input_name}"


file_loader = FileSystemLoader(templates_dir())

environment = TypedEnvironment(loader=file_loader)

async_environment = TypedEnvironment(enable_async=True, loader=file_loader)



import os
import pandas as pd
from pydantic import BaseModel, Field
from io import StringIO

class DataWriter:
    def __init__(self, data, file_path="", write_options=None):
        print("init " + file_path)

        if write_options is None:
            write_options = {}
        self.file_path = file_path
        
        # Determine file extension
        _, file_extension = os.path.splitext(self.file_path)
        file_extension = file_extension.lower()

        # Handle different data formats
        if file_extension == '.csv':
            if isinstance(data, list) and all(isinstance(item, dict) for item in data):
                self.df = pd.DataFrame(data)
            elif isinstance(data, dict) and all(isinstance(v, list) for v in data.values()):
                self.df = pd.DataFrame(data)
            else:
                raise ValueError("For CSV files, data must be a list of dictionaries or a dictionary of lists.")
            
            # Check for any variation of 'ID' column
            id_column = next((col for col in self.df.columns if col.lower() == 'id'), None)
            
            # If no ID column exists, add it
            if id_column is None:
                self.df.insert(0, 'ID', range(len(self.df)))
            else:
                # If ID column exists but is not the first column, move it to the front
                if self.df.columns.get_loc(id_column) != 0:
                    cols = self.df.columns.tolist()
                    cols.insert(0, cols.pop(cols.index(id_column)))
                    self.df = self.df[cols]
        elif file_extension == '.md':
            if isinstance(data, str):
                self.md_content = data
            else:
                raise ValueError("For Markdown files, data must be a string.")
        else:
            raise ValueError(f"Unsupported file type: {file_extension}")

        self.write_options = write_options

    def get_file_path(self):
        context_generator = DataFrameContextGenerator()
        context_string = context_generator.generate_context(self.df) if hasattr(self, 'df') else ""

        inst = FileNameModel.to_inst("Create a filename that fits \n" + context_string)
        return inst.file_name

    def forward(self, **kwargs):
        if not self.file_path:
            self.file_path = self.get_file_path()

        _, file_extension = os.path.splitext(self.file_path)
        file_extension = file_extension.lower()

        if file_extension == '.csv':
            print("write " + self.file_path)
            try:
                # Set index=False to prevent writing the index as a separate column
                self.df.to_csv(self.file_path, index=False, **self.write_options)
            except Exception as e:
                raise ValueError(f"Failed to write to {self.file_path} due to: {e}")

        elif file_extension == '.md':
            print("write " + self.file_path)
            try:
                with open(self.file_path, 'w') as file:
                    file.write(self.md_content)
            except Exception as e:
                raise ValueError(f"Failed to write to {self.file_path} due to: {e}")

        else:
            raise ValueError(f"Unsupported file type: {file_extension}")

class DataFrameContextGenerator(BaseModel):
    descriptive_stats: bool = True
    dtypes_info: bool = True
    context: str = ""

    class Config:
        arbitrary_types_allowed = True

    def generate_context(self, df) -> str:
        buffer = StringIO()
        df.info(buf=buffer)
        info_str = buffer.getvalue()

        context_parts = []

        if self.descriptive_stats:
            desc_stats = df.describe().to_string()
            context_parts.append(desc_stats)

        if self.dtypes_info:
            dtypes_str = df.dtypes.to_string()
            context_parts.append(dtypes_str)

        context = "\n".join(context_parts)
        self.context = context
        return self.context

class FileNameModel(BaseModel):
    file_name: str = Field(..., description="Unique CSV filename based on the data provided.")
    extension: str = Field("csv", description="File extension for the output file.")

def main():
    # Test case 1: No ID column (should add 'ID' column)
    data_csv1 = [
        {'Book Title': 'The Great Gatsby', 'Author': 'F. Scott Fitzgerald', 'Price': 10.99, 'Sold Copies': 500},
        {'Book Title': '1984', 'Author': 'George Orwell', 'Price': 9.99, 'Sold Copies': 800},
        {'Book Title': 'Brave New World', 'Author': 'Aldous Huxley', 'Price': 8.99, 'Sold Copies': 650},
        {'Book Title': 'The Catcher in the Rye', 'Author': 'J.D. Salinger', 'Price': 12.99, 'Sold Copies': 450}
    ]
    writer_csv1 = DataWriter(file_path="test1_no_id.csv", data=data_csv1)
    writer_csv1.forward()

    # Test case 2: 'id' column exists (should keep existing 'id' and move to front)
    data_csv2 = [
        {'id': 'A1', 'Book Title': 'To Kill a Mockingbird', 'Author': 'Harper Lee', 'Price': 11.99, 'Sold Copies': 750},
        {'id': 'B2', 'Book Title': 'Pride and Prejudice', 'Author': 'Jane Austen', 'Price': 7.99, 'Sold Copies': 950},
    ]
    writer_csv2 = DataWriter(file_path="test2_existing_id.csv", data=data_csv2)
    writer_csv2.forward()

    # Test case 3: 'ID' column exists but not at the front (should move to front)
    data_csv3 = [
        {'Book Title': 'The Hobbit', 'ID': 'C3', 'Author': 'J.R.R. Tolkien', 'Price': 14.99, 'Sold Copies': 1000},
        {'Book Title': 'Dune', 'ID': 'D4', 'Author': 'Frank Herbert', 'Price': 13.99, 'Sold Copies': 850},
    ]
    writer_csv3 = DataWriter(file_path="test3_id_not_front.csv", data=data_csv3)
    writer_csv3.forward()

    # Test case 4: 'Id' column exists (should keep existing 'Id' and move to front)
    data_csv4 = [
        {'Book Title': 'The Alchemist', 'Author': 'Paulo Coelho', 'Id': 'E5', 'Price': 9.99, 'Sold Copies': 1200},
        {'Book Title': 'The Da Vinci Code', 'Author': 'Dan Brown', 'Id': 'F6', 'Price': 12.99, 'Sold Copies': 1100},
    ]
    writer_csv4 = DataWriter(file_path="test4_Id_exists.csv", data=data_csv4)
    writer_csv4.forward()

    # Example Usage for Markdown (unchanged)
    data_md = "# Book List\n\n- The Great Gatsby\n- 1984\n- Brave New World\n- The Catcher in the Rye"
    writer_md = DataWriter(file_path="Tetris_Blog_Phi3Med.md", data=data_md)
    writer_md.forward()

if __name__ == "__main__":
    main()



"""
Advanced LLM-Powered Chat Tools for Sungen

This module provides state-of-the-art language model interactions for the Sungen project,
pushing the boundaries of what's possible with LLMs in a development environment.

Key Features and Capabilities:
1. Multi-Modal Understanding: Processes and generates text, code, images, and audio.
2. Temporal Awareness: Maintains long-term memory and understands time-based context.
3. Adaptive Learning: Continuously improves based on user interactions and feedback.
4. Ethical AI Integration: Implements robust ethical checks and bias detection.
5. Quantum-Inspired Language Processing: Utilizes quantum computing concepts for enhanced NLP.
6. Neuro-Symbolic Reasoning: Combines neural networks with symbolic AI for improved reasoning.
7. Federated Learning: Enables privacy-preserving distributed model updates.
8. Explainable AI (XAI): Provides detailed explanations for its decision-making process.
9. Emotion and Sentiment Analysis: Detects and responds to user emotions and sentiments.
10. Multilingual and Cross-Cultural Understanding: Seamlessly operates across languages and cultures.
11. Code Evolution Tracking: Analyzes and suggests improvements based on project history.
12. Adversarial Robustness: Implements advanced techniques to resist adversarial attacks.
13. Automated Formal Verification: Generates and verifies formal proofs for critical code sections.
14. Quantum-Resistant Cryptography Integration: Ensures future-proof security in communications.
15. Biocomputing Interface: Explores potential biological computing paradigms for enhanced processing.

Advanced Interaction Modes:
- Socratic Dialogue: Engages in deep, philosophical discussions to enhance understanding.
- Counterfactual Reasoning: Explores alternative scenarios and their implications.
- Metacognitive Analysis: Reflects on its own thought processes and decision-making.
- Interdisciplinary Synthesis: Combines knowledge from various fields for novel insights.
- Temporal Extrapolation: Predicts future trends and potential outcomes in software development.
- Quantum Entanglement Simulation: Uses quantum-inspired algorithms for complex problem-solving.
- Neuroplasticity Emulation: Adapts its neural architecture based on the task at hand.

Ethical Considerations:
- Implements advanced fairness algorithms to ensure unbiased responses.
- Continuously monitors and mitigates potential negative societal impacts.
- Adheres to strict privacy protocols, including differential privacy techniques.
- Provides transparency reports on decision-making processes and data usage.

Future-Ready Features:
- Brain-Computer Interface (BCI) Compatibility: Prepared for direct neural interactions.
- Quantum Supremacy Adaptation: Ready to leverage quantum computing breakthroughs.
- Artificial General Intelligence (AGI) Sandbox: Explores the boundaries of general AI capabilities.
- Singularity Preparedness Protocol: Implements safeguards for potential technological singularity.

Note: Some features may be conceptual or in early experimental stages. Always refer to the
latest documentation for current capabilities and ethical guidelines.

Usage:
    from sungen.utils.chat_tools import chatbot, handle_global_error, set_global_exception_handler

    # Initialize the advanced chatbot
    response = chatbot("Explain quantum computing in the context of NLP", context="Advanced AI research")

    # Set up global error handling with advanced AI-powered analysis
    set_global_exception_handler()

For more information on ethical AI and advanced LLM capabilities, visit:
https://www.nature.com/articles/s41586-021-03819-2
"""
import dspy
import typer
import traceback
import sys
from sungen.utils.dspy_tools import init_dspy, init_ol, GPT_DEFAULT_MODEL

class ChatbotAssistance(dspy.Signature):
    """
    Provides guidance and assistance to users in developing projects with sungen,
    leveraging the integrated chatbot functionality.
    """
    question = dspy.InputField(desc="The user's query or request for assistance.")
    context = dspy.InputField(desc="Background information relevant to the user's query.")
    conversation_history = dspy.InputField(desc="Previous exchanges between the user and the chatbot, if any.")
    answer = dspy.OutputField(desc="The chatbot's response to the user's query.")

class ErrorAnalysis(dspy.Signature):
    """
    Analyzes uncaught errors and provides expert-level suggestions for fixing them.
    """
    error_message = dspy.InputField(desc="The error message and traceback.")
    error_context = dspy.InputField(desc="Additional context about where the error occurred.")
    analysis = dspy.OutputField(desc="Expert analysis of the error.")
    fix_suggestion = dspy.OutputField(desc="Detailed suggestion for fixing the error.")

def chatbot(question, context, history="", model=GPT_DEFAULT_MODEL):
    """
    An advanced chatbot for interacting with the Sungen project and its plugins.

    Chat Rules and Advanced Functionality:
    1. Context Awareness: The chatbot maintains context across multiple interactions within a session.
    2. Code Generation: Can generate Python code snippets for Sungen plugins and commands.
    3. Explanation Mode: Prefix your question with "Explain:" for detailed explanations of concepts or code.
    4. Comparison Mode: Use "Compare: X vs Y" to get a detailed comparison between two features or concepts.
    5. Example Mode: Start with "Example:" to request practical examples of usage or implementation.
    6. Debug Mode: Begin with "Debug:" followed by an error message or problematic code for analysis and suggestions.
    7. Refactor Mode: Use "Refactor:" followed by code to get suggestions on improving code quality or performance.
    8. CLI Help: Ask about specific CLI commands for detailed usage information.
    9. Plugin Interaction: Inquire about creating, modifying, or using Sungen plugins.
    10. Version Compatibility: Can provide information about compatibility between different versions of Sungen or its plugins.
    11. Best Practices: Ask for best practices in Sungen development, plugin creation, or Python programming in general.
    12. Integration Queries: Can assist with questions about integrating Sungen with other tools or frameworks.
    13. Configuration Help: Provide guidance on configuring Sungen or its plugins for different environments.
    14. Troubleshooting: Offer step-by-step troubleshooting advice for common issues.
    15. Feature Explanation: Detailed explanations of Sungen features and their use cases.

    Args:
        question (str): The user's query or request for assistance.
        context (str): Background information relevant to the user's query.
        history (str, optional): Previous exchanges between the user and the chatbot. Defaults to "".
        model (str, optional): The language model to use for generating responses. Defaults to GPT_DEFAULT_MODEL.

    Returns:
        str: The chatbot's response history, including the latest answer.

    Note:
        The chatbot uses advanced natural language processing to understand and respond to queries.
        It can handle a wide range of topics related to Sungen, from basic usage to advanced development concepts.
    """
    if not question:
        question = typer.prompt("How can I help you?")

    # Initialize the appropriate model
    if model.startswith("gpt"):
        lm = init_dspy(model=model)
    else:
        lm = init_ol(model=model)

    # Include the docstring in the context
    # full_context = f"""
    # Chatbot Capabilities and Rules:
    # {chatbot.__doc__}

    # User Context:
    # {context}
    # """
    full_context = f"""

    User Context:
    {context}
    """

    qa = dspy.ChainOfThought(ChatbotAssistance)
    response = qa(question=question, context=full_context, conversation_history=history).answer
    history += response
    print(f"Chatbot: {response}")
    print("(Press Ctrl+C to close)")

    while True:
        want = typer.prompt("How can I help more?")
        response = qa(question=want, context=full_context, conversation_history=history).answer
        history += response
        print(f"Chatbot: {response}")
        print("(Press Ctrl+C to close)")

    return history

def handle_global_error(error_type, error_value, tb, model=GPT_DEFAULT_MODEL):
    """
    Handles global uncaught errors, provides the error message, and offers LLM-generated tips and ideas to fix.
    """
    error_message = f"Error Type: {error_type.__name__}\nError Message: {str(error_value)}\n\nTraceback:\n{''.join(traceback.format_tb(tb))}"
    error_context = f"File: {tb.tb_frame.f_code.co_filename}\nLine: {tb.tb_lineno}\nFunction: {tb.tb_frame.f_code.co_name}"

    print("An unexpected error occurred:")
    print(error_message)
    print("\nError Context:")
    print(error_context)

    print("\nAnalyzing the error and generating suggestions...")

    # Initialize the appropriate model
    if model.startswith("gpt"):
        lm = init_dspy(model=model)
    else:
        lm = init_ol(model=model)

    error_analyzer = dspy.ChainOfThought(ErrorAnalysis)
    analysis_result = error_analyzer(error_message=error_message, error_context=error_context)

    print("\n--- Error Analysis ---")
    print(analysis_result.analysis)
    print("\n--- Fix Suggestions ---")
    print(analysis_result.fix_suggestion)

# Set up the global exception handler
def set_global_exception_handler(model=GPT_DEFAULT_MODEL):
    def global_exception_handler(error_type, error_value, tb):
        handle_global_error(error_type, error_value, tb, model)
    
    sys.excepthook = global_exception_handler



"""project"""
import typer


app = typer.Typer()


@app.command(name="project")
def _project():
    """project"""
    typer.echo("Running project subcommand.")
     
@app.command(name="init")
def project_init():
    """init"""
    typer.echo("Running init subcommand.")
 
@app.command(name="config")
def project_config():
    """config"""
    typer.echo("Running config subcommand.")
 
@app.command(name="rollback")
def project_rollback():
    """rollback"""
    typer.echo("Running rollback subcommand.")
 
@app.command(name="status")
def project_status():
    """status"""
    typer.echo("Running status subcommand.")



import dspy

from sungen.dsl.dsl_pydantic_models import StepDSLModel, PipelineDSLModel
from sungen.retrievers.data_retriever import DataRetriever


def _get_retrieval_model_instance(pipeline: PipelineDSLModel, step: StepDSLModel):
    """
    Get the retrieval model instance for a given step from the top level definition.
    """
    if not step.retrievers_model:
        return None

    rm_label = step.retrievers_model

    # Find the rm class within the dspy module. Need to import the class dynamically from the dspy module
    rm_config = next((m for m in pipeline.retrievers_models if m.label == rm_label), None)

    if rm_config.name == "DataRetriever":
        return DataRetriever(**rm_config.args)

    if rm_config and hasattr(dspy, rm_config.name):
        rm_class = getattr(dspy, rm_config.name)
        rm_inst = rm_class(**rm_config.args)
        return rm_inst



"""issue"""
import typer
from sungen.utils.git_tools import github

app = typer.Typer()

@app.command(name="create")
def issue_create(
    title: str = typer.Option(..., "--title", "-t", help="Title of the issue"),
    body: str = typer.Option("", "--body", "-b", help="Body of the issue"),
    repo: str = typer.Option(..., "--repo", "-r", help="Repository name (owner/repo)")
):
    """Create a new issue in a GitHub repository."""
    gh = github()
    repo_obj = gh.get_repo(repo)
    issue = repo_obj.create_issue(title=title, body=body)
    typer.echo(f"Created issue #{issue.number}: {issue.title}")
    typer.echo(f"URL: {issue.html_url}")

@app.command(name="close")
def issue_close(
    number: int = typer.Option(..., "--number", "-n", help="Issue number to close"),
    repo: str = typer.Option(..., "--repo", "-r", help="Repository name (owner/repo)")
):
    """Close an existing issue in a GitHub repository."""
    gh = github()
    repo_obj = gh.get_repo(repo)
    issue = repo_obj.get_issue(number=number)
    issue.edit(state="closed")
    typer.echo(f"Closed issue #{issue.number}: {issue.title}")

@app.command(name="list")
def issue_list(
    repo: str = typer.Option(..., "--repo", "-r", help="Repository name (owner/repo)"),
    state: str = typer.Option("open", "--state", "-s", help="State of issues to list (open, closed, all)")
):
    """List issues in a GitHub repository."""
    gh = github()
    repo_obj = gh.get_repo(repo)
    issues = repo_obj.get_issues(state=state)

    typer.echo(f"Issues in {repo} ({state}):")
    for issue in issues:
        typer.echo(f"#{issue.number}: {issue.title} - {issue.state}")

@app.command(name="comment")
def issue_comment(
    number: int = typer.Option(..., "--number", "-n", help="Issue number to comment on"),
    body: str = typer.Option(..., "--body", "-b", help="Comment body"),
    repo: str = typer.Option(..., "--repo", "-r", help="Repository name (owner/repo)")
):
    """Add a comment to an existing issue in a GitHub repository."""
    gh = github()
    repo_obj = gh.get_repo(repo)
    issue = repo_obj.get_issue(number=number)
    comment = issue.create_comment(body)
    typer.echo(f"Added comment to issue #{issue.number}")
    typer.echo(f"Comment URL: {comment.html_url}")

@app.command(name="update")
def issue_update(
    number: int = typer.Option(..., "--number", "-n", help="Issue number to update"),
    title: str = typer.Option(None, "--title", "-t", help="New title for the issue"),
    body: str = typer.Option(None, "--body", "-b", help="New body for the issue"),
    state: str = typer.Option(None, "--state", "-s", help="New state for the issue (open or closed)"),
    repo: str = typer.Option(..., "--repo", "-r", help="Repository name (owner/repo)")
):
    """Update an existing issue in a GitHub repository."""
    gh = github()
    repo_obj = gh.get_repo(repo)
    issue = repo_obj.get_issue(number=number)
    
    update_params = {}
    if title:
        update_params["title"] = title
    if body:
        update_params["body"] = body
    if state:
        update_params["state"] = state
    
    issue.edit(**update_params)
    typer.echo(f"Updated issue #{issue.number}: {issue.title}")
    typer.echo(f"URL: {issue.html_url}")

@app.command(name="assign")
def issue_assign(
    number: int = typer.Option(..., "--number", "-n", help="Issue number to assign"),
    assignee: str = typer.Option(..., "--assignee", "-a", help="GitHub username to assign the issue to"),
    repo: str = typer.Option(..., "--repo", "-r", help="Repository name (owner/repo)")
):
    """Assign an issue to a GitHub user."""
    gh = github()
    repo_obj = gh.get_repo(repo)
    issue = repo_obj.get_issue(number=number)
    issue.add_to_assignees(assignee)
    typer.echo(f"Assigned issue #{issue.number} to {assignee}")

@app.command(name="labels")
def issue_labels(
    number: int = typer.Option(..., "--number", "-n", help="Issue number to modify labels"),
    add: list[str] = typer.Option([], "--add", help="Labels to add to the issue"),
    remove: list[str] = typer.Option([], "--remove", help="Labels to remove from the issue"),
    repo: str = typer.Option(..., "--repo", "-r", help="Repository name (owner/repo)")
):
    """Add or remove labels from an issue."""
    gh = github()
    repo_obj = gh.get_repo(repo)
    issue = repo_obj.get_issue(number=number)
    
    if add:
        issue.add_to_labels(*add)
    if remove:
        issue.remove_from_labels(*remove)

    typer.echo(f"Updated labels for issue #{issue.number}")

    issue = repo_obj.get_issue(number=number)
    
    typer.echo(f"Current labels: {', '.join([label.name for label in issue.labels])}")
    


"""
togaflss Plugin
This plugin provides Generates code for a TOGAF-based solution, integrating Design for Lean Six Sigma principles to optimize architecture framework for Fortune 10 companies, enhancing efficiency and reducing waste..

Author: Sean Chatman
Version: 1.0.0
"""
import typer


app = typer.Typer(name="togaflss", help="Generates code for a TOGAF-based solution, integrating Design for Lean Six Sigma principles to optimize architecture framework for Fortune 10 companies, enhancing efficiency and reducing waste.")


@app.command()
def example():
    """
    Example command for togaflss.
    
    Usage:
    sungen togaflss example
    """
    typer.echo("This is an example command for the togaflss plugin.")


def check_installation():
    """
    Check if togaflss dependencies are installed and install them if not present.
    """
    import importlib.util
    import subprocess
    import sys

    try:
        if importlib.util.find_spec("togaflss_dep") is None:
            print("togaflss_dep is not installed. Installing...")
            subprocess.check_call([sys.executable, "-m", "pip", "install", "togaflss_dep"])
            print("togaflss_dep has been installed successfully.")

    except Exception as e:
        print(f"Error during library installation check")
        raise e


def register_plugin(parent_app: typer.Typer):
    """Register the plugin with the main application."""
    # check_installation()

    parent_app.add_typer(app, name="togaflss")


"""
interpolation Plugin
This plugin provides This plugin performs text interpolation..

Author: Sean Chatman
Version: 1.0.0
"""

import typer
app = typer.Typer(name="interpolation")


@app.command()
def example(source_doc: str, pipeline_yaml: str, prompt: str):
    """Example command for interpolation.

    This command executes a pipeline to convert a Saltcorn ebook into a Sungen ebook format.

    Args:
        source_doc (str): The path to the source Saltcorn ebook file (e.g., 'path/to/saltcorn_ebook.epub').
        pipeline_yaml (str): The path to the YAML file defining the pipeline (e.g., 'path/to/pipeline.yaml').
        prompt (str): The prompt to guide the conversion process (e.g., 'Convert this Saltcorn ebook into a Sungen ebook format.').

    Example usage:
        $ python -m sungen.plugins.interpolation.interpolation_plugin example \
            --source_doc path/to/saltcorn_ebook.epub \
            --pipeline_yaml path/to/pipeline.yaml \
            --prompt "Convert this Saltcorn ebook into a Sungen ebook format."
    """
    # Call the execute_pipeline function with the provided arguments
    from sungen.dsl.dsl_pipeline_executor import execute_pipeline
    context = execute_pipeline(pipeline_yaml, {"source_doc": source_doc, "prompt": prompt})
    
    typer.echo("Pipeline executed with context:")
    typer.echo(context)


def register_plugin(parent_app: typer.Typer):
    """Register the plugin with the main application."""
    parent_app.add_typer(app)


"""optimize"""
import typer


app = typer.Typer()


@app.command(name="optimize")
def _optimize():
    """optimize"""
    typer.echo("Running optimize subcommand.")
     
@app.command(name="apply-triz")
def optimize_apply_triz():
    """apply-triz"""
    typer.echo("Running apply_triz subcommand.")
 
@app.command(name="performance")
def optimize_performance():
    """performance"""
    typer.echo("Running performance subcommand.")
 
@app.command(name="resource-usage")
def optimize_resource_usage():
    """resource-usage"""
    typer.echo("Running resource_usage subcommand.")
 
@app.command(name="security")
def optimize_security():
    """security"""
    typer.echo("Running security subcommand.")



import os
import subprocess
import re
import yaml  # Add this import

from sungen.plugins.aider.devlog.summary_config import SummaryConfig, FileSource  # Import FileSource


def write_with_aider(chapter):
    """Create a markdown file using the aider command."""
    command = [
        'aider',
        '--file', chapter.path,
        '--yes',
        '--message', f"Create a markdown file for '{chapter.title}' at {chapter.path} with suitable initial content based on its name and provided text.",
        '--', 'CONVENTIONS.md', 'SUMMARY.md', 'TRANSCRIPT.md', 'press_release.md'
    ]
    print(command)
    subprocess.run(command)


def write_chapters(config: SummaryConfig):
    """Write chapters using the aider command based on the configuration."""
    for chapter in config.prefix_chapters + config.numbered_chapters + config.suffix_chapters:
        # Use the chapter object directly
        if chapter.path and not os.path.exists(chapter.path):
            write_with_aider(chapter)  # Pass the entire chapter object
            print(f"Created chapter: {chapter.title} at {chapter.path}")  # Log chapter creation

        # Process subchapters
        for subchapter in chapter.subchapters:
            if subchapter.path and not os.path.exists(subchapter.path):
                write_with_aider(subchapter)  # Pass the entire subchapter object
                print(f"Created subchapter: {subchapter.title} at {subchapter.path}")  # Log subchapter creation


def main():
    # Navigate to the directory containing the markdown files
    os.chdir('/Users/sac/dev/sungen/src/sungen/plugins/aider/devlog')

    # Load the summary configuration using FileSource
    config = SummaryConfig(config_sources=FileSource(file='summary_config.yaml'))  # Initialize SummaryConfig with loaded data

    # Write chapters based on the configuration
    write_chapters(config)


if __name__ == "__main__":
    main()



import sys
from dataclasses import dataclass
from typing import Any, Optional


@dataclass
class CommandContext:
    model: str = "gpt-4-0613"
    input: Optional[str] = None
    output: Optional[str] = None
    in_n_out: Optional[str] = None
    prompt: Optional[str] = None
    verbose: bool = False
    no_copy: bool = False
    paste: bool = False
    example: Optional[str] = None
    template: Optional[str] = None
    clear_history: bool = False
    auto_output: bool = False
    text: Optional[str] = ""
    auto_summarize: int = 4
    tokens: Optional[str] = None
    extension: Optional[str] = None
    response: Optional[str] = ""
    dsl: Optional[str] = None
    system_prompt: Optional[str] = None
    schema: Optional[str] = None
    append: Optional[str] = None


def create_context(params: dict[str, Any]) -> CommandContext:
    if str(params.get("model")) == "3":
        params["model"] = "gpt-3.5-turbo-0613"

    if str(params.get("model")) == "4":
        params["model"] = "gpt-4-0613"

    in_n_out = params.get("in_n_out")
    if in_n_out:
        params["input"] = in_n_out
        params["output"] = in_n_out

    # Add standard input to text
    # if sys.stdin.read():
    #     params['text'] += sys.stdin.read()

    if params.get("text") is None:
        params["text"] = ""

    return CommandContext(**params)



import importlib

import dspy

from sungen.dsl.dsl_pydantic_models import RMModuleDSLModel
from sungen.retrievers.data_retriever import DataRetriever


def _create_rm_module_from_model(module_model: RMModuleDSLModel) -> dspy.Module:
    """
    Create a DSPy Module from a ModuleDSLModel instance.
    """


def _load_rm_module_class(dspy_module_class_name: str):
    """
    Dynamically loads a signature class by its fully qualified name.
    """
    module_name, class_name = dspy_module_class_name.rsplit('.', 1)
    module = importlib.import_module(module_name)
    return getattr(module, class_name)


def _get_rm_module_instance(pipeline, rendered_args, step):
    """
    Get the module instance for a given step from the top level definition or load the module.
    Uses the DSLModule class from sungen.dspy_modules.dsl_module to handle dspy_modules defined in the pipeline YAML.
    """
    # Get the file_path from the pipeline.context or rendered_args
    # file_path = pipeline.context.get("file_path", rendered_args.get("file_path"))

    return DataRetriever(pipeline=pipeline, step=step, **rendered_args)





import sqlite3
from pathlib import Path

import dspy
import pandas as pd
import os

from pandasql import sqldf


# Function to apply an SQL query to a DataFrame
def apply_sql_to_dataframe(df, query):
    """
    This function applies an SQL query to the given DataFrame.

    Parameters:
    - df: pandas DataFrame.
    - query: SQL query as a string.

    Returns:
    - DataFrame containing the result of the SQL query.
    """
    # Dynamically add the DataFrame to the local scope with the name 'df'
    local_env = locals()
    local_env["df"] = df
    return sqldf(query, local_env)


def read_any(filepath, query, read_options=None):
    if read_options is None:
        read_options = {}
    _, file_extension = os.path.splitext(filepath)
    file_extension = file_extension.lower()
    read_functions = {
        '.csv': pd.read_csv,
        '.xls': pd.read_excel,
        '.xlsx': pd.read_excel,
        '.pickle': pd.read_pickle,
        '.pkl': pd.read_pickle,
        '.h5': pd.read_hdf,
        '.hdf': pd.read_hdf,
        '.sql': pd.read_sql,  # Connection argument needed here.
        '.db': pd.read_sql,  # Connection argument needed here.
        '.json': pd.read_json,
        '.parquet': pd.read_parquet,
        '.orc': pd.read_orc,
        '.feather': pd.read_feather,
        '.gbq': pd.read_gbq,  # Requires additional args like project_id.
        '.html': pd.read_html,  # Returns a list of DataFrames.
        '.xml': pd.read_xml,
        '.stata': pd.read_stata,
        '.sas': pd.read_sas,  # Might require additional arguments.
        '.sav': pd.read_spss,
        '.dta': pd.read_stata,
        '.fwf': pd.read_fwf,  # Might require formatting arguments.
    }

    if file_extension == '.sql' or file_extension == '.db':
        connection = sqlite3.connect(filepath)
        # Updated to pass 'params' from read_options to pd.read_sql_query
        df = pd.read_sql_query(query, connection, params=read_options.get('params', None))
        connection.close()
        return df

    if file_extension in read_functions:
        read_func = read_functions[file_extension]
        try:
            return read_func(filepath, **read_options)
        except Exception as e:
            raise ValueError(f"Failed to read {filepath} due to: {e}")
    else:
        raise ValueError(f"Unsupported file type: {file_extension}")


class DataRetriever(dspy.Retrieve):
    supported_extensions = ['.csv', '.xls', '.xlsx', '.pickle', '.pkl', '.h5', '.hdf', 
                            '.sql', '.db', '.json', '.parquet', '.orc', '.feather', 
                            '.gbq', '.html', '.xml', '.stata', '.sas', '.sav', '.dta', '.fwf']

    def __init__(self, file_path: str | Path, query: str = "", return_columns=None,
                 read_options=None, pipeline=None, step=None, **kwargs):
        super().__init__()
        if return_columns is None:
            return_columns = []
        if read_options is None:
            read_options = {}

        self.pipeline = pipeline
        self.step = step

        self.file_path = str(file_path)
        self.return_columns = return_columns
        self.read_options = read_options

        # Read the data using the read_any function
        self.df = read_any(file_path, query, read_options)  # No SQL query here

    @classmethod
    def supports_file_type(cls, file_extension: str) -> bool:
        return file_extension.lower() in cls.supported_extensions

    def forward(self, query: str = None, k: int = None, **kwargs) -> list[dict]:
        # Check if a SQL query is provided
        if query:
            # Apply the SQL query to the DataFrame
            matches = apply_sql_to_dataframe(self.df, query)
        else:
            matches = self.df

        if k is not None:
            matches = matches.head(k)

        if self.return_columns:
            # Ensure only specified return columns are included in the results
            matches = matches[self.return_columns]

        result = matches.to_dict(orient='records')

        if self.pipeline:
            self.pipeline.context.data = result

        return result


def main():
    file_path = 'sample_data.csv'
    query = "SELECT name FROM df WHERE age < 30"  # SQL query to filter data
    return_columns = ['name']

    # Initialize the DataRetriever with the path to your CSV file
    # Note: The query passed here is an SQL query that will be applied to the DataFrame.
    # If your initial reading does not require filtering, you might pass an empty string or adjust accordingly.
    data_retriever = DataRetriever(file_path=file_path, return_columns=return_columns)

    # For demonstration, applying the SQL query using the forward method
    # Normally, you'd integrate your SQL query logic where it fits best in your application flow
    filtered_results = data_retriever.forward(query=query)

    # Print the filtered results
    print("Filtered Results:")
    print(filtered_results)


if __name__ == "__main__":
    main()



import dspy
from pathlib import Path
from fnmatch import fnmatch

from dspy import Prediction

from sungen.utils.dspy_tools import init_ol, init_dspy
import wcmatch.glob as wglob

import dspy
import typer


class CodeRetriever(dspy.Retrieve):
    def __init__(self, path, gitignore=None, max_depth=None):
        super().__init__()
        self.path = Path(path)
        self.max_depth = max_depth
        self.gitignore = Path(gitignore) if gitignore else self.path / ".gitignore"
        self.gitignore_patterns = self.parse_gitignore(self.gitignore)
        self.gitignore_patterns.add(".git")

    def parse_gitignore(self, gitignore_path):
        if not gitignore_path.exists():
            return set()

        with gitignore_path.open("r", encoding="utf-8") as file:
            patterns = set(
                line.strip() for line in file if line.strip() and not line.startswith("#")
            )
        return patterns

    def forward(self, query = None) -> list[str] | Prediction | list[Prediction]:
        content = []
        file_dict = {}

        # Use wcmatch.WcMatch to search for files
        flags = wglob.GLOBSTAR | wglob.IGNORECASE

        # Setup the walker with respect to the max_depth
        for file_path in wglob.iglob(
                str(self.path / "**"),
                flags=flags,
                exclude=self.gitignore_patterns,
                depth=(0 if self.max_depth is None else self.max_depth)
        ):
            file_path = Path(file_path)

            # Apply additional filters
            if (
                    file_path.is_file()
                    and not self.is_binary(file_path)
                    and (not query or self.is_filtered(file_path, query))
            ):
                try:
                    print(file_path)
                    with file_path.open("r", encoding="utf-8") as f:
                        file_content = f.read()
                        file_dict[file_path] = file_content
                except UnicodeDecodeError:
                    continue

                file_info = self.extract_file_info(file_path)
                content.append(file_info + file_content + "\n```\n\n")

        return dspy.Prediction(passages=content, file_dict=file_dict)

    def is_filtered(self, file_path, query):
        # Use wcmatch.fnmatch to check if file matches the query
        return wglob.globmatch(str(file_path), query)

    def is_binary(self, file_path):
        try:
            with open(file_path, "rb") as file:
                return b"\x00" in file.read(1024)
        except IOError:
            return False

    def extract_file_info(self, file_path):
        file_extension = file_path.suffix.lstrip('.')
        file_info = f"## File: {file_path}\n\n```{file_extension}\n"
        return file_info


def get_files_from_directory(directory, query, gitignore=None, max_depth=None):
    """Retrieves code snippets from a specified directory using CodeRetriever."""
    code_retriever = CodeRetriever(directory, gitignore, max_depth)
    result = code_retriever.forward(query)
    return result.passages  # Return the list of file contents


def main(directory: str, query: str, gitignore: str = None, max_depth: int = None):
    # Initialize OpenAI LLM and DSPy settings
    init_ol(model="phi3", max_tokens=2000)

    # Instantiate CodeRetriever
    code_retriever = CodeRetriever(directory, gitignore, max_depth)

    # Retrieve and print results
    result = code_retriever.forward(query)
    for file_content in result.passages:
        print(file_content)


if __name__ == '__main__':
    typer.run(main)

# def main():
#     init_ol(model="phi3", max_tokens=2000)
#     #init_dspy(model="gpt-4o", max_tokens=3000)
#     path = "/Users/sac/dev/nuxt-ai-chatbot/stores"
#     gitignore = "/.gitignore"  # Optional
#
#     code_retriever = CodeRetriever(path, gitignore)
#     result = code_retriever.forward("*.py") # and only the code after ```python and ending with ``` ")
#     print(result.passages)
#
#     for file_content in result.passages:
#         #from sungen.dspy_modules.nuxt_module import nuxt_call
#         print(file_content)
#         #nuxt = nuxt_call(path=path, readme=file_content)
#         #print(nuxt)
# for file_content in result.passages:
#     print(file_content)  # Here, you can instead write to a Markdown file or process further.

# If I want one file containing all the code snippets
# with open("code_snippets.md", "w") as f:
#     for file_content in result.passages:
#         f.write(file_content)


# if __name__ == '__main__':
#     main()


import os
from pathlib import Path
from datetime import datetime, timedelta
from typing import List, Any
import typer

from sungen.utils.dspy_tools import init_ol

# Initialize Typer application
app = typer.Typer()


def query_python_files(directory: str, gitignore: str = None) -> List[str]:
    """Find all Python files in the directory."""
    return get_files_from_directory(directory, "*.py", gitignore)


def query_markdown_files(directory: str, gitignore: str = None) -> List[str]:
    """Find all Markdown files in the directory."""
    return get_files_from_directory(directory, "*.md", gitignore)


def query_large_files(directory: str, size_threshold: int, gitignore: str = None) -> List[str]:
    """Find all files larger than a specified size."""
    result = []
    for file_path in wglob.iglob(str(Path(directory) / "**"), flags=wglob.GLOBSTAR | wglob.RECURSIVE):
        if Path(file_path).is_file() and (os.path.getsize(file_path) > size_threshold):
            result.append(str(file_path))
    return result


def query_recent_files(directory: str, days: int, gitignore: str = None) -> List[str]:
    """Find files modified within a specific number of days."""
    cutoff_time = datetime.now() - timedelta(days=days)
    result = []
    for file_path in wglob.iglob(str(Path(directory) / "**"), flags=wglob.GLOBSTAR | wglob.RECURSIVE):
        if Path(file_path).is_file() and datetime.fromtimestamp(os.path.getmtime(file_path)) > cutoff_time:
            result.append(str(file_path))
    return result


def query_ignored_files(directory: str, gitignore: str = None) -> List[str]:
    """List all files that match patterns in .gitignore."""
    code_retriever = CodeRetriever(directory, gitignore)
    return [str(path) for path in code_retriever.gitignore_patterns]


def query_binary_files(directory: str, gitignore: str = None) -> List[str]:
    """Find binary files in the directory."""
    result = []
    code_retriever = CodeRetriever(directory, gitignore)
    for file_path in wglob.iglob(str(Path(directory) / "**"), flags=wglob.GLOBSTAR | wglob.RECURSIVE):
        if Path(file_path).is_file() and code_retriever.is_binary(Path(file_path)):
            result.append(str(file_path))
    return result


def query_files_with_pattern(directory: str, pattern: str, gitignore: str = None) -> List[str]:
    """Find files containing a specific text pattern."""
    result = []
    code_retriever = CodeRetriever(directory, gitignore)
    for file_path in wglob.iglob(str(Path(directory) / "**"), flags=wglob.GLOBSTAR | wglob.RECURSIVE):
        if Path(file_path).is_file() and pattern in Path(file_path).read_text(encoding="utf-8", errors="ignore"):
            result.append(str(file_path))
    return result


def main(directory: str, query_type: str, size: int = 1024, days: int = 7, pattern: str = "", gitignore: str = None):
    # Initialize LLM configuration
    init_ol(model="phi3", max_tokens=2000)

    if query_type == "python":
        results = query_python_files(directory, gitignore)
    elif query_type == "markdown":
        results = query_markdown_files(directory, gitignore)
    elif query_type == "large":
        results = query_large_files(directory, size, gitignore)
    elif query_type == "recent":
        results = query_recent_files(directory, days, gitignore)
    elif query_type == "ignored":
        results = query_ignored_files(directory, gitignore)
    elif query_type == "binary":
        results = query_binary_files(directory, gitignore)
    elif query_type == "pattern":
        results = query_files_with_pattern(directory, pattern, gitignore)
    else:
        print(f"Unknown query type: {query_type}")
        return

    for result in results:
        print(result)


if __name__ == "__main__":
    typer.run(main)



import dspy
from sungen.utils.dspy_tools import init_dspy

class TextToDFSQLSignature(dspy.Signature):
    """
    Converts natural language text and a DataFrame schema into an SQL query
    compatible with pandasql.sqldf, where the DataFrame is referenced as 'df'.

    ```python
    import pandas as pd
    from pandasql import sqldf

    # Example DataFrame
    df = pd.DataFrame({
        'name': ['Alice', 'Bob', 'Charlie'],
        'age': [25, 30, 35]
    })

    # Generated SQL query from the signature
    query = "SELECT * FROM df WHERE age > 30"

    # Execute query with sqldf
    result = sqldf(query, locals())
    ```
    """
    text = dspy.InputField(desc="Natural language text describing the query.")
    df_schema = dspy.InputField(desc="Schema of the DataFrame, containing columns and their data types.")
    df_data = dspy.InputField(desc="Data of the DataFrame, containing rows and columns.")
    sql_query = dspy.OutputField(
        desc=(
            "Generated SQL query based on the input text, designed to be compatible with "
            "pandasql.sqldf. The query will reference the DataFrame as 'df' and use SQLite-compatible "
            "SQL syntax. Ensure the SQL operations and functions used are supported by SQLite"
            "ALWAYS SELECT *."
        ),
        prefix="```sql\nSELECT * FROM df "
    )

class DFSQLModule(dspy.Module):
    """DFSQLModule"""

    def __init__(self, **forward_args):
        super().__init__()
        self.forward_args = forward_args
        self.output = None

    def forward(self, text, df_schema, df_data):
        # Use the custom Signature class for prediction
        pred = dspy.Predict(TextToDFSQLSignature)
        self.output = "SELECT * FROM df " + pred(text=text, df_schema=df_schema, df_data=df_data).sql_query
        self.output = self.output.replace("```", "").strip()
        return self.output


def dfsql_call(text, df_schema, df_data):
    text_to_data_frame_sql_generator = DFSQLModule()
    return text_to_data_frame_sql_generator.forward(text=text, df_schema=df_schema, df_data=df_data)


def main():
    init_dspy()
    # app = RemindersApp()
    # app.export_reminders("reminders.csv")
    # dr = DataRetriever(file_path="reminders.csv")
    # df_schema = dr.df.columns.tolist()
    # df_data = dr.df.values.tolist()
    #
    # text = "Find what am I supposed to cut?"
    # result = dfsql_call(text=text, df_schema=df_schema, df_data=df_data)
    #
    # results = app.query(result)
    #
    # print(results[0])


if __name__ == "__main__":
    main()



from importlib import import_module
from pathlib import Path
from github import Github

import typer
import os

def source_dir(path):
    return os.path.join(os.path.dirname(os.path.dirname(__file__)), path)

def load_commands(app, cmd_dir):
    cmd_dir = Path(cmd_dir) if isinstance(cmd_dir, str) else cmd_dir

    try:
        package_path = cmd_dir.resolve()
        module_base = ".".join(package_path.parts[-2:])
    except IndexError:
        raise ValueError(f"The path '{cmd_dir}' does not appear to be within an importable package structure.")

    for filepath in cmd_dir.glob("*_cmd.py"):
        module_name = f"{module_base}.{filepath.stem}"
        module = import_module(module_name)
        if hasattr(module, "app"):
            app.add_typer(module.app, name=filepath.stem[:-4], help=module.__doc__)

def get_github_instance(token: str = typer.Option(None, envvar="GITHUB_TOKEN")) -> Github:
    if not token:
        typer.echo("Error: No GitHub token provided and $GITHUB_TOKEN is not set.", err=True)
        raise typer.Exit(code=1)

    return Github(token)
